from unittest.mock import patch

import pandas as pd
import pytest

from refinitiv.data.content.search import Views as SearchViews
from refinitiv.data.discovery import search_templates
from refinitiv.data.discovery._search_templates.search import DiscoverySearchTemplate
from refinitiv.data.discovery._search_templates.manage import UserNamespace
from refinitiv.data import get_config


class Tmp:
    """Tpl template

    Methods
    -------
    search
        a
            A param
            default: 1

        b
            B param

        c
            default: 'string'
    """


class TestDiscoverySearchTemplate:
    def test_repr(self):
        assert (
            repr(DiscoverySearchTemplate(name="DST"))
            == "<DiscoverySearchTemplate 'DST'>"
        )

    def test_export(self):
        class TestSearchTemplate(DiscoverySearchTemplate):
            def __init__(self):
                super().__init__(
                    name="RICCategory",
                    pass_through_defaults={
                        "view": SearchViews.QUOTES_AND_STIRS,
                        "select": "RCSAssetCategoryLeaf",
                        "top": 1,
                    },
                    placeholders_defaults={
                        "ric": "ABC",
                    },
                    filter="RIC eq '#{ric}'",
                )

        assert TestSearchTemplate()._export() == {
            "request_body": {
                "View": "QuotesAndSTIRs",
                "Select": "RCSAssetCategoryLeaf",
                "Top": 1,
                "Filter": "RIC eq '#{ric}'",
            },
            "parameters": {
                "ric": {"default": "ABC"},
            },
        }


class TestNamespaces:
    def test_none_namespace(self):
        get_config().set_param(
            "search.templates", {"CustomNamespace1": None}, auto_create=True
        )
        assert isinstance(search_templates["CustomNamespace1"], UserNamespace)

    def test_namespace_getitem(self):
        get_config().set_param(
            "search.templates",
            {"CustomNamespace2": {"InternalNamespace": None}},
            auto_create=True,
        )
        assert isinstance(
            search_templates["CustomNamespace2"]["InternalNamespace"], UserNamespace
        )

    def test_namespace_keys(self):
        get_config().set_param(
            "search.templates",
            {"CustomNamespace3": {"InternalNamespace": None}},
            auto_create=True,
        )
        assert search_templates["CustomNamespace3"].keys() == ["InternalNamespace"]

    def test_namespace_in(self):
        get_config().set_param(
            "search.templates",
            {"CustomNamespace4": {"InternalNamespace": None}},
            auto_create=True,
        )
        assert "InternalNamespace" in search_templates["CustomNamespace4"]

    def test_template_in_namespace(self):
        get_config().set_param(
            "search.templates",
            {
                "CustomNamespace5": {
                    "Equity": {
                        "request_body": {
                            "View": "EquityQuotes",
                            "Filter": "#{a}",
                        },
                    }
                }
            },
            auto_create=True,
        )
        assert isinstance(
            search_templates["CustomNamespace5"]["Equity"], DiscoverySearchTemplate
        )

    def test_namespace_with_body(self):
        get_config().set_param(
            "search.templates",
            {
                "CustomNamespace6": {
                    "parameters": {
                        "ric": {"default": "ABC"},
                    },
                }
            },
            auto_create=True,
        )
        assert isinstance(search_templates["CustomNamespace6"], UserNamespace)

    def test_template_in_nested_namespace(self):
        get_config().set_param(
            "search.templates",
            {
                "CustomNamespace7": {
                    "InternalNamespace": {
                        "Equity": {
                            "request_body": {
                                "View": "EquityQuotes",
                                "Filter": "#{a}",
                            },
                        }
                    }
                }
            },
            auto_create=True,
        )
        assert isinstance(
            search_templates["CustomNamespace7"]["InternalNamespace"]["Equity"],
            DiscoverySearchTemplate,
        )

    def test_request_body_namespace_key(self):
        get_config().set_param(
            "search.templates",
            {"CustomNamespace8": None},
            auto_create=True,
        )
        with pytest.raises(KeyError) as excinfo:
            template = search_templates["CustomNamespace8"]["request_body"]
            assert "request_body is a reserved key" in str(excinfo.value)


class TestTemplates:
    def test_keys(self):
        config = get_config()
        if "search.templates" in config:
            del config["search.templates"]
        config.set_param("search.templates", {"Equity": None}, auto_create=True)
        assert set(search_templates.keys()) == {"Equity"}

    def test_empty_config(self):
        config = get_config()
        if "search.templates" in config:
            del config["search.templates"]
        assert set(search_templates.keys()) == set()
        with pytest.raises(KeyError):
            search_templates["Abracadabra"]

    def test_in_operator(self):
        get_config().set_param("search.templates", {"Equity": None}, auto_create=True)
        assert "Equity" in search_templates

    def test_converts_arguments_case(self):
        get_config().set_param(
            "search.templates",
            {
                "Equity": {
                    "request_body": {
                        "GroupCount": 5,
                    }
                }
            },
            auto_create=True,
        )
        assert "group_count" in search_templates["Equity"]._search_kwargs()
        assert search_templates["Equity"]._search_kwargs()["group_count"] == 5

    def test_defaults(self):
        get_config().set_param(
            "search.templates",
            {
                "Equity": {
                    "request_body": {
                        "View": "EquityQuotes",
                        "Filter": "#{a}",
                    },
                    "parameters": {"a": {"default": 1}},
                }
            },
            auto_create=True,
        )
        search_templates["Equity"]._search_kwargs()

    def test_definition_arg_in_parameters(self):
        get_config().set_param(
            "search.templates",
            {
                "Equity": {
                    "request_body": {
                        "View": "EquityQuotes",
                    },
                    "parameters": {
                        "top": {
                            "default": 100,
                            "description": "top description",
                        }
                    },
                }
            },
            auto_create=True,
        )
        assert search_templates["Equity"]._search_kwargs().get("top") == 100
        assert "top description" in search_templates["Equity"].__doc__

    def test_docstrings(self):
        get_config().set_param(
            "search.templates",
            {
                "Tpl": {
                    "request_body": {
                        "View": "EquityQuotes",
                        "Filter": "#{a} #{b} #{c}",
                    },
                    "description": "Tpl template",
                    "parameters": {
                        "a": {"default": 1, "description": "A param"},
                        "b": {"description": "B param"},
                        "c": {"default": "string"},
                    },
                }
            },
            auto_create=True,
        )

        assert search_templates["Tpl"].__doc__.strip() == Tmp.__doc__.strip()

    def test_conditionals(self):
        get_config().set_param(
            "search.templates",
            {
                "Roots": {
                    "request_body": {
                        "Filter": "{{if ric}}RIC eq '#{ric}'{{endif}}",
                    },
                    "parameters": {
                        "ric": {
                            "optional": True,
                        }
                    },
                }
            },
            auto_create=True,
        )
        assert search_templates["Roots"]._search_kwargs() == {"filter": ""}

    def test_locals(self):
        get_config().set_param(
            "search.templates",
            {
                "Roots": {
                    "request_body": {
                        "Filter": "RicRoot eq '#{_.Root.RicRoot.0}'",
                    },
                    "locals": {"Root": {"request_body": {"Filter": "RIC eq '#{ric}'"}}},
                }
            },
            auto_create=True,
        )
        with patch(
            "refinitiv.data.discovery._search_templates.search.search.Definition.get_data"
        ) as mock:
            mock.return_value.data.df = pd.DataFrame([{"RicRoot": "1333"}])
            assert (
                search_templates["Roots"]._search_kwargs(ric="1333.T")["filter"]
                == "RicRoot eq '1333'"
            )

    def test_locals_join(self):
        get_config().set_param(
            "search.templates",
            {
                "Roots": {
                    "request_body": {"Filter": "RicRoot in [#{_.Root.RicRoot|in}]"},
                    "locals": {"Root": {"request_body": {"Filter": "RIC eq '#{ric}'"}}},
                }
            },
            auto_create=True,
        )
        with patch(
            "refinitiv.data.discovery._search_templates.search.search.Definition.get_data"
        ) as mock:
            mock.return_value.data.df = pd.DataFrame(
                [{"RicRoot": "1333"}, {"RicRoot": "1334"}]
            )
            assert (
                search_templates["Roots"]._search_kwargs(ric="1333.T")["filter"]
                == "RicRoot in ['1333', '1334']"
            )

    def test_locals_required_sub_arg(self):
        get_config().set_param(
            "search.templates",
            {
                "Roots": {
                    "request_body": {"Filter": "RicRoot eq '#{_.Root.RicRoot}'"},
                    "locals": {"Root": {"request_body": {"Filter": "RIC eq '#{ric}'"}}},
                }
            },
            auto_create=True,
        )
        with patch(
            "refinitiv.data.discovery._search_templates.search.search.Definition.get_data"
        ):
            with pytest.raises(KeyError):
                search_templates["Roots"]._search_kwargs()

    def test_unknown_subtemplate(self):
        get_config().set_param(
            "search.templates",
            {
                "RootsUnknownSubtemplate": {
                    "request_body": {"Filter": "RicRoot eq '#{_.Root.RicRoot}'"},
                    "locals": {},
                }
            },
            auto_create=True,
        )
        with pytest.raises(ValueError):
            search_templates["RootsUnknownSubtemplate"]._search_kwargs()

    def test_locals_nonused(self):
        get_config().set_param(
            "search.templates",
            {
                "Roots": {
                    "request_body": {"Filter": "RicRoot eq 'ABC'"},
                    "locals": {"Root": {"request_body": {}}},
                }
            },
            auto_create=True,
        )
        with patch(
            "refinitiv.data.discovery._search_templates.search.search.Definition.get_data"
        ) as mock:
            assert (
                search_templates["Roots"]._search_kwargs()["filter"]
                == "RicRoot eq 'ABC'"
            )
            mock.assert_not_called()

    def test_locals_only_in_if_detected(self):
        get_config().set_param(
            "search.templates",
            {
                "RootsWithIf": {
                    "request_body": {
                        "Filter": "{{if _.Shmoot.RicRoot.0}}RicRoot eq 'USD='{{endif}}"
                    },
                    "locals": {
                        "Shmoot": {"request_body": {"Filter": "RIC eq '#{ric}'"}}
                    },
                }
            },
            auto_create=True,
        )
        with patch(
            "refinitiv.data.discovery._search_templates.search.search.Definition.get_data"
        ):
            with pytest.raises(KeyError):
                search_templates["RootsWithIf"]._search_kwargs()



import re

import allure
import pytest

from refinitiv.data.content.ownership.consolidated import top_n_concentration
from refinitiv.data.errors import RDError
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ownership.helpers import (
    check_response_dataframe_instrument_contains_expected_universes,
    get_expected_column,
)
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_response_status,
    check_non_empty_response_data,
    check_response_dataframe_contains_columns_names,
)

EXPECTED_COLUMN_NAMES = [
    "Instrument",
    "Total Shareholders Count",
    "% SharesOutstanding",
    "SharesHeld",
    "SharesHeld Value",
]

EXPECTED_COLUMN_TITLES = [
    "instrument",
    "Total Shareholders Count",
    "% SharesOutstanding",
    "SharesHeld",
    "SharesHeld Value",
]


@allure.suite("Content object - Ownership Consolidated Top-n-Concentration")
@allure.feature("Content object - Ownership Fund Top-n-Concentration")
@allure.severity(allure.severity_level.CRITICAL)
class TestFundTopNConcentration:
    @allure.title(
        "Create Consolidated Top-n-Concentration definition object with valid required params and get data"
    )
    @pytest.mark.caseid("37556970")
    @pytest.mark.parametrize(
        "universe,count,use_field_names_in_headers",
        [
            ("ONEX.CCP", 5, True),
            ("US02079K1079", 1, False),
            (["TRI.N", "ONEX.CCP", "AAPL.O"], 20, True),
            (["ONEX.CCP", "invalid universe"], 100, False),
        ],
    )
    def test_create_consolidated_top_n_concentration_object_with_valid_fields_and_get_data(
        self, open_platform_session, universe, count, use_field_names_in_headers
    ):
        expected_column = get_expected_column(
            use_field_names_in_headers, EXPECTED_COLUMN_TITLES, EXPECTED_COLUMN_NAMES
        )
        response = top_n_concentration.Definition(
            universe, count, use_field_names_in_headers
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column)
        check_response_dataframe_instrument_contains_expected_universes(
            response, universe
        )

    @allure.title(
        "Create asunc Consolidated Top-n-Concentration definition object with mix valid and invalid params"
    )
    @pytest.mark.caseid("37556971")
    @pytest.mark.parametrize(
        "universe,invalid_universe,count",
        [
            (["ONEX.CCP", "invalid universe"], "invalid universe", 15),
        ],
    )
    @pytest.mark.smoke
    async def test_async_consolidated_top_n_concentration_object_with_mix_valid_and_invalid_params(
        self, open_platform_session_async, universe, invalid_universe, count
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            top_n_concentration.Definition(
                universe, use_field_names_in_headers=True, count=count
            ),
            top_n_concentration.Definition(
                invalid_universe, use_field_names_in_headers=True, count=count
            ),
        )
        check_response_status(
            response=valid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=None,
            expected_error_message="Failed to resolve identifiers ['invalid universe']",
        )
        check_non_empty_response_data(valid_response)
        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED_TWELVE,
            expected_error_message="Unable to resolve all requested identifiers. Requested items: ['invalid universe']",
        )
        check_response_dataframe_contains_columns_names(
            valid_response, EXPECTED_COLUMN_NAMES
        )
        check_response_dataframe_instrument_contains_expected_universes(
            valid_response, universe
        )

    @allure.title(
        "Create Consolidated Top-n-Concentration definition object with invalid params and get error"
    )
    @pytest.mark.caseid("37556972")
    @pytest.mark.parametrize(
        "universe,count", [("", 30), ("ONEX.CCP", -200), ([""], "invalid count")]
    )
    def test_create_consolidated_top_n_concentration_object_with_invalid_fields_and_get_error(
        self, open_platform_session, universe, count
    ):
        if type(count) == int and count < 1:
            with pytest.raises(RDError) as error:
                top_n_concentration.Definition(universe, count).get_data()
            assert (
                str(error.value)
                == "Error code 400 | Validation error: validation failure list:\ncount in query should be greater than or equal to 1"
            )
        elif type(count) == int and count >= 1:
            with pytest.raises(RDError) as error:
                top_n_concentration.Definition(universe, count).get_data()
            assert (
                str(error.value)
                == "Error code 400 | Validation error: validation failure list:\nuniverse.0 in query should be at least 1 chars long"
            )
        else:
            with pytest.raises(
                TypeError,
                match=re.escape(
                    "Parameter 'count' of invalid type provided: 'str', expected types: ['int']"
                ),
            ):
                top_n_concentration.Definition(universe, count).get_data()



from functools import partial

import refinitiv.data.content as rdc
from refinitiv.data.content.ipa.surfaces import cap
from refinitiv.data.content.ipa.surfaces import fx
from tests.unit.conftest import StubResponse

NEWS_STORY_DEFINITION = rdc.news.story.Definition(
    "urn:newsml:reuters.com:20201026:nPt6BSyBh"
)
NEWS_STORY_RESPONSE = StubResponse(
    {
        "newsItem": {
            "_conformance": "power",
            "_guid": "tag:reuters.com,2022-04-08:newsml_RTV7TbbdW",
            "_standard": "NewsML-G2",
            "_standardversion": "2.18",
            "_version": 5,
            "catalogRef": [
                {
                    "_href": "http://xml.media.reuters.com/g2-standards/catalogs/ReutersMedia_G2-Standards-Catalog_v1.xml"
                }
            ],
            "rightsInfo": [
                {
                    "copyrightHolder": {"_literal": "Thomson Reuters"},
                    "copyrightNotice": [
                        {
                            "$": "(c) Copyright Thomson Reuters 2022. Click For Restrictions - http://about.reuters.com/fulllegal.asp"
                        }
                    ],
                }
            ],
            "itemMeta": {
                "itemClass": {"_qcode": "ninat:text", "_rtr:msgType": "A"},
                "provider": {"_literal": "reuters.com"},
                "versionCreated": {"$": "2022-04-08T06:02:25.633Z"},
                "firstCreated": {"$": "2022-04-08T06:00:00.972Z"},
                "pubStatus": {"_qcode": "stat:usable"},
                "role": {"_qcode": "itemRole:N"},
                "title": [
                    {
                        "$": "Refinitiv Newscasts - Scholz lobt Schulterschluss bei Kriegsflüchtlingen"
                    }
                ],
                "signal": [{"_qcode": "edStat:C"}],
                "expires": [{"$": "2023-05-08T06:02:25.633Z"}],
                "link": [
                    {
                        "_rel": "irel:seeAlso",
                        "_residref": "urn:newsml:reuters.com:20220408:nRTV7TbbdW",
                    }
                ],
                "itemMetaExtProperty": [
                    {
                        "_creator": "rftResRef:sys26",
                        "_id": "cse1",
                        "_rel": "extCptRel:hasMduMinorVersion",
                        "_value": "0.1",
                        "_valuedatatype": "xs:string",
                        "related": [
                            {
                                "_rel": "extCptRel:hasRelatedTimestamp",
                                "_value": "2022-04-08T06:02:25.694",
                                "_valuedatatype": "xs:dateTime",
                            }
                        ],
                    }
                ],
                "rtr:versionedId": [
                    {"_guid": "tag:reuters.com,2022-04-08:newsml_RTV7TbbdW:5"}
                ],
            },
            "contentMeta": {
                "urgency": {"$": 3},
                "infoSource": [
                    {"_qcode": "NS:RTRS", "_role": "cRole:source"},
                ],
                "creator": [{"_qcode": "NS:RTRS", "_role": "cRole:source"}],
                "contributor": [{"_qcode": "NS:RTRS", "_role": "cRole:enhancer"}],
                "audience": [{"_qcode": "NP:RITV"}],
                "altId": [
                    {"_type": "idType:USN", "$": "RTV7TbbdW"},
                ],
                "language": [{"_tag": "de"}],
                "subject": [
                    {
                        "_confidence": 100,
                        "_creator": "rftResRef:sys26",
                        "_how": "howextr:tool",
                        "_id": "S1",
                        "_qcode": "G:1",
                        "_why": "why:inferred",
                        "related": [
                            {
                                "_creator": "rftResRef:sys24",
                                "_how": "howextr:tool",
                                "_id": "grp0",
                                "_qcode": "hmlInd:high",
                                "_rel": "extCptRel:hasRelevanceGroup",
                                "_why": "why:inferred",
                            }
                        ],
                    },
                ],
                "headline": [
                    {
                        "_dir": "ltr",
                        "_xml:lang": "de",
                        "$": "Refinitiv Newscasts - Scholz lobt Schulterschluss bei Kriegsflüchtlingen",
                    }
                ],
                "contentMetaExtProperty": [
                    {
                        "_creator": "rftResRef:sys26",
                        "_how": "howextr:tool",
                        "_id": "DK1",
                        "_qcode": "dedupeKey:strict:20220408_-78987acd6c0b959f0dbcca11ea11464c110268e4",
                        "_rel": "extCptRel:hasDedupeKey",
                        "_why": "why:inferred",
                        "related": [
                            {
                                "_creator": "rftResRef:sys26",
                                "_how": "howextr:tool",
                                "_qcode": "dedupeKeyType:strict",
                                "_rel": "extCptRel:hasDedupeKeyType",
                                "_why": "why:inferred",
                            }
                        ],
                    },
                ],
            },
            "assert": [
                {"_qcode": "NP:RITV", "type": [{"_qcode": "cptType:15"}]},
            ],
            "derivedFrom": [
                {"_idrefs": "A1", "_qcode": "NP:RITV"},
            ],
            "contentSet": {
                "inlineXML": [
                    {
                        "_contenttype": "application/xhtml+xml",
                        "$": '<html lang="en"xml:lang="en"xmlns="http://www.w3.org/1999/xhtml"><head ><title >Refinitiv Newscasts - Scholz lobt Schulterschluss bei Kriegsflüchtlingen</title></head><body ><div class="storyframe"id="storydiv"><div class="TEXT"id="storybody"name="storybody"xmlns:custExtension="urn:xslExtensions"xmlns:xsi="http://www.w3.org/2001/XMLSchema/"><div id="storybodycontent0"><span class="storycontent"><pre >Click the following link to watch video: <a href="https://share.newscasts.refinitiv.com/link?entryId=1_bonf89p1&referenceId=tag:reuters.com,2022:newsml_OV039108042022RP1_996&pageId=RefinitivNewscasts">https://share.newscasts.refinitiv.com/link?entryId=1_bonf89p1&referenceId=tag:reuters.com,2022:newsml_OV039108042022RP1_996&pageId=RefinitivNewscasts</a></pre><a class="storylink_item"href=""/>Source: Thomson Reuters<br /><br />Description: Die Einigung von Bund und Ländern sei eine gute Grundlage für den Umgang mit den Ukraine-Flüchtlingen, sagt der Bundeskanzler.<br />Short Link: <a href="https://refini.tv/3jfkLXx">https://refini.tv/3jfkLXx</a><br /><br />Video Transcript:<br /><br />Verified transcript not available</span></div></div></div></body></html>',
                    }
                ],
                "inlineData": [
                    {
                        "_contenttype": "text/plain",
                        "_dir": "ltr",
                        "_xml:lang": "de",
                        "$": "\nClick the following link to watch video: https://share.newscasts.refinitiv.com/link?entryId=1_bonf89p1&referenceId=tag:reuters.com,2022:newsml_OV039108042022RP1_996&pageId=RefinitivNewscasts\nSource: Thomson Reuters\n\nDescription: Die Einigung von Bund und Ländern sei eine gute Grundlage für\nden Umgang mit den Ukraine-Flüchtlingen, sagt der Bundeskanzler.\nShort Link: https://refini.tv/3jfkLXx\n\nVideo Transcript:\n\nVerified transcript not available\n",
                    }
                ],
            },
        },
        "topNewsMetadata": [],
    }
)

NEWS_STORY_RESPONSE_DATA = StubResponse(
    {
        "story": {
            "headlineHtml": '<h1 class="storyHeadline">\n<span class="headline">Nasdaq cae por controles de exportación EEUU a China que afectan a semiconductores</span> -\n<span class="source" title="RTRS">Reuters</span>\n</h1>',
            "storyHtml": '<div class="storyContent" lang="es"><style type="text/css">.storyContent * {border-color:inherit !important;outline-color:inherit !important;}</style><p class="tr-advisory">.</p><p class="tr-by">Por Shreyashi Sanyal y Bansari Mayur Kamdar</p><p class="tr-story-p1"><span class="tr-dateline">10 oct (Reuters)</span><span class="tr-dl-sep"> - </span>El índice Nasdaq caía el lunes hasta tocar mínimos de dos años, ya que los fabricantes de semiconductores se llevaban la peor parte de los esfuerzos de Estados Unidos para obstaculizar la industria de chips de China, mientras los inversores operaban con cautela antes del comienzo de la temporada de resultados. </p><p>* El índice SE Semiconductor de Filadelfia <a href="reuters://REALTIME/Verb=FullQuote/ric=.SOX" data-type="ric" data-ric=".SOX" translate="no" dir="ltr">.SOX</a> bajó un 2,7%, después de que el Gobierno del presidente Joe Biden publicó un amplio conjunto de controles de exportación el viernes, incluida una medida para aislar a China de ciertos chips semiconductores fabricados en cualquier parte del mundo con equipos estadounidenses.</p><p>* Algunos de los componentes más importantes del índice, incluida Nvidia Corp. <a href="reuters://REALTIME/Verb=FullQuote/ric=NVDA.O" data-type="ric" data-ric="NVDA.O" translate="no" dir="ltr">NVDA.O</a>, Qualcomm Inc. <a href="reuters://REALTIME/Verb=FullQuote/ric=QCOM.O" data-type="ric" data-ric="QCOM.O" translate="no" dir="ltr">QCOM.O</a> y Micron Technology Inc. <a href="reuters://REALTIME/Verb=FullQuote/ric=MU.O" data-type="ric" data-ric="MU.O" translate="no" dir="ltr">MU.O</a> perdieron entre 1,3% y 3,3% al inicio de la sesión.</p><p>* En las operaciones de media mañana, el Promedio Industrial Dow Jones <a href="reuters://REALTIME/Verb=FullQuote/ric=.DJI" data-type="ric" data-ric=".DJI" translate="no" dir="ltr">.DJI</a> subía 54,93 puntos, o un 0,19%, a 29.351,72 unidades; mientras que el S&amp;P 500 <a href="reuters://REALTIME/Verb=FullQuote/ric=.SPX" data-type="ric" data-ric=".SPX" translate="no" dir="ltr">.SPX</a> bajaba 9,31 puntos, o un 0,26%, a 3.630,35 unidades; y el índice Nasdaq Composite <a href="reuters://REALTIME/Verb=FullQuote/ric=.IXIC" data-type="ric" data-ric=".IXIC" translate="no" dir="ltr">.IXIC</a> cedía 73,99 puntos, o un 0,69%, a 10.578,41 unidades.</p><p>* Los gigantes tecnológicos Apple Inc. <a href="reuters://REALTIME/Verb=FullQuote/ric=AAPL.O" data-type="ric" data-ric="AAPL.O" translate="no" dir="ltr">AAPL.O</a> y Microsoft Corp. <a href="reuters://REALTIME/Verb=FullQuote/ric=MSFT.O" data-type="ric" data-ric="MSFT.O" translate="no" dir="ltr">MSFT.O</a> caían un 0,9% y un 1,5%, respectivamente, lastrando el subíndice del sector tecnológico del S&amp;P 500 <a href="reuters://REALTIME/Verb=FullQuote/ric=.SPLRCT" data-type="ric" data-ric=".SPLRCT" translate="no" dir="ltr">.SPLRCT</a> .</p><p><br/></p><p><br/></p><p class="tr-signoff"> (Reportes de Ankika Biswas y Shreyashi Sanyal en Bengaluru; reporte adicional de Bansari Mayur Kamdar. Editado en español por Marion Giraldo)</p><p class="tr-contactinfo">((Mesa de edición en español +562 24374447. Twitter: <a href="https://twitter.com/ReutersLatam" data-type="url" class="tr-link" translate="no">https://twitter.com/ReutersLatam</a>; ))</p><div class="tr-additinfo tr-desktop-part"><pre>REUTERS MG/\n</pre></div><p class="line-break"><br/></p><p class="tr-copyright">(c) Copyright Thomson Reuters 2022. Click For Restrictions - https://agency.reuters.com/en/copyright.html</p><p class="line-break"><br/></p><p class="tr-slugline">Keywords: MERCADOS-WALLST/ (MEDIA)</p></div>',
            "storyInfoHtml": '<h5 style="direction:ltr"><span data-version-created-date="2022-10-10T15:25:07.000Z" class="releasedDate">10-Oct-2022 15:25:07</span></h5>',
        }
    }
)

FUNDAMENTAL_AND_REFERENCE_DEFINITION = rdc.fundamental_and_reference.Definition(
    ["IBM"], ["TR.Volume"]
)
FUNDAMENTAL_AND_REFERENCE_RESPONSE_DESKTOP = StubResponse(
    {
        "responses": [
            {
                "columnHeadersCount": 1,
                "data": [["IBM", 3143309]],
                "headerOrientation": "horizontal",
                "headers": [
                    [
                        {"displayName": "Instrument"},
                        {"displayName": "Volume", "field": "TR.VOLUME"},
                    ]
                ],
                "rowHeadersCount": 1,
                "totalColumnsCount": 2,
                "totalRowsCount": 2,
            }
        ]
    }
)
FUNDAMENTAL_AND_REFERENCE_RESPONSE_PLATFORM = StubResponse(
    {
        "links": {"count": 1},
        "variability": "",
        "universe": [
            {
                "Instrument": "IBM",
                "Company Common Name": "International Business Machines Corp",
                "Organization PermID": "4295904307",
                "Reporting Currency": "USD",
            }
        ],
        "data": [["IBM", 3538317]],
        "messages": {
            "codes": [[-1, -1]],
            "descriptions": [{"code": -1, "description": "ok"}],
        },
        "headers": [
            {
                "name": "instrument",
                "title": "Instrument",
                "type": "string",
                "description": "The requested Instrument as defined by the user.",
            },
            {
                "name": "TR.Volume",
                "title": "Volume",
                "type": "number",
                "decimalChar": ".",
                "description": "Volume for the latest trading day.",
            },
        ],
    }
)

NEWS_HEADLINES_DEFINITION = rdc.news.headlines.Definition("Refinitiv")
NEWS_HEADLINES_RESPONSE_UDF = StubResponse(
    {
        "headlines": [
            {
                "displayDirection": "LeftToRight",
                "documentType": "Story",
                "firstCreated": "2022-04-08T11:19:38.000Z",
                "isAlert": False,
                "language": "L:en",
                "reportCode": "",
                "sourceCode": "NS:RFT",
                "sourceName": "Refinitiv",
                "storyId": "urn:newsml:reuters.com:20220408:nIfp955dm0:3",
                "text": "CARE ratings for Indian debt instruments-Apr 8",
                "versionCreated": "2022-04-08T14:23:23.661Z",
            },
        ],
        "newer": "/headlines?payload=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",
        "older": "/headlines?payload=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",
    }
)
NEWS_HEADLINES_RESPONSE = StubResponse(
    {
        "data": [
            {
                "storyId": "urn:newsml:reuters.com:20220408:nRTV7TbbdW:5",
                "newsItem": {
                    "_version": 5,
                    "contentMeta": {
                        "audience": [{"_qcode": "NP:RITV"}],
                        "creator": [{"_qcode": "NS:RTRS", "_role": "sRole:source"}],
                        "infoSource": [
                            {"_qcode": "NS:RTRS", "_role": "sRole:source"},
                            {"_qcode": "NS:RTRS", "_role": "sRole:origProv"},
                        ],
                        "language": [{"_tag": "de"}],
                        "subject": [
                            {"_qcode": "M:2CS"},
                            {"_qcode": "M:2CU"},
                            {"_qcode": "M:1Q1"},
                            {"_qcode": "G:A"},
                            {"_qcode": "M:2CM"},
                            {"_qcode": "M:2CQ"},
                            {"_qcode": "M:1QD"},
                            {"_qcode": "M:2CR"},
                            {"_qcode": "M:2CT"},
                            {"_qcode": "G:1"},
                            {"_qcode": "M:2CV"},
                            {"_qcode": "G:71"},
                            {"_qcode": "G:B"},
                            {"_qcode": "M:2CN"},
                            {"_qcode": "M:2CP"},
                            {"_qcode": "M:2D1"},
                        ],
                        "urgency": {"$": 3},
                    },
                    "itemMeta": {
                        "firstCreated": {"$": "2022-04-08T06:00:00.972Z"},
                        "versionCreated": {"$": "2022-04-08T06:02:25.633Z"},
                        "title": [
                            {
                                "$": "Refinitiv Newscasts - Scholz lobt Schulterschluss bei Kriegsflüchtlingen"
                            }
                        ],
                    },
                },
            },
        ],
        "meta": {
            "count": 10,
            "pageLimit": 10,
            "next": "H4sIAAAAAAAAABXNwQ6CMBAE0H/Zq5iUWhDqWb1hQogX46GBRZvUlrSlYgj/bnvc2cmbFUapPFrgK/jfhMBhtIgeFw8ZBKHmFLU4Si29DDEblXg54A94oxiU1AjPLYOPWHozaw88JxlMMhifSI1fJ4co5CWrD1XN2LEuKn0/N7SoypzsKKGUMFKeyL69dJGXrknbHLyN01F2xqYzSp25qSFWLCoMQvd4tWae4k8oBdsfCNrjlMoAAAA=",
            "prev": "H4sIAAAAAAAAABXOTQ6CMBAF4LvMVkzKj6B4AHea1EYXxkWBqTYZW9IWxBDublnOm5dvZgalKaCDeobw6xFqUA4x4BQggVHSsEYclTY66DFmiuTLQ/2AN8qOtEF4Lgl85NTawQSoU5ZAr0cbVtLg1+suCmlZHPJDVRW7Ms8NF7dKNE1332Qsy1jB9ke25YJfo6/9eT0e/5DkEaLtrVvnaAl7oS52HBKO0rR4cnbo404SwfIHSJxlH8wAAAA=",
        },
    }
)

SURFACE_SWAPTION_DEFINITION = rdc.ipa.surfaces.swaption.Definition(
    surface_tag="My EUR VolCube",
    underlying_definition=rdc.ipa.surfaces.swaption.SwaptionSurfaceDefinition(
        instrument_code="EUR",
        discounting_type=rdc.ipa.surfaces.swaption.DiscountingType.OIS_DISCOUNTING,
    ),
    surface_parameters=rdc.ipa.surfaces.swaption.SwaptionCalculationParams(
        shift_percent=3,
        x_axis=rdc.ipa.surfaces.swaption.Axis.STRIKE,
        y_axis=rdc.ipa.surfaces.swaption.Axis.TENOR,
        z_axis=rdc.ipa.surfaces.swaption.Axis.EXPIRY,
    ),
    surface_layout=rdc.ipa.surfaces.swaption.SurfaceLayout(
        format=rdc.ipa.surfaces.swaption.Format.MATRIX,
    ),
)

SURFACE_SWAPTION_RESPONSE = StubResponse(
    {
        "data": [
            {
                "surfaceTag": "My EUR VolCube",
                "surface": [
                    [
                        None,
                        "-2.00",
                        "-1.50",
                        "-1.00",
                    ],
                    [
                        "1",
                        "124.63",
                        "108.79",
                        "91.51",
                    ],
                    [
                        "2",
                        "157.25",
                        "139.29",
                        "120.61",
                    ],
                ],
            }
        ]
    }
)

SURFACES_DEFINITION = rdc.ipa.surfaces.Definitions(
    [
        SURFACE_SWAPTION_DEFINITION,
    ]
)

SURFACES_RESPONSE = SURFACE_SWAPTION_RESPONSE

PRICING_DEFINITION = rdc.pricing.Definition("EUR=")
PRICING_RESPONSE = StubResponse(
    [
        {
            "ID": 754,
            "Type": "Refresh",
            "Key": {"Service": "ERT_FD3_LF1", "Name": "EUR="},
            "State": {
                "Stream": "NonStreaming",
                "Data": "Ok",
                "Text": "**All is well",
            },
            "Qos": {
                "Timeliness": "Realtime",
                "Rate": "TimeConflated",
                "RateInfo": 3000,
            },
            "PermData": "AwD9Umw=",
            "SeqNumber": 38494,
            "Fields": {
                "PROD_PERM": 526,
                "RDNDISPLAY": 153,
            },
        }
    ]
)

PRICING_DF_CREATING_RESPONSE = StubResponse(
    [
        {
            "ID": 207,
            "Type": "Refresh",
            "Key": {"Service": "ERT_FD3_LF1", "Name": "EUR="},
            "State": {"Stream": "NonStreaming", "Data": "Ok", "Text": "**All is well"},
            "Qos": {
                "Timeliness": "Realtime",
                "Rate": "TimeConflated",
                "RateInfo": 3000,
            },
            "PermData": "AwD9Umw=",
            "SeqNumber": 50430,
            "Fields": {
                "CF_NAME": "Euro",
                "DDS_DSO_ID": 12348,
                "BR_LINK1": None,
                "QUOTIM_2": "10:30:40",
                "QUOTE_DT2": "2022-06-22",
                "ASKHI1_MS": "22:55:25.876",
            },
        }
    ]
)

FORWARD_CURVE_RESPONSE = StubResponse(
    {
        "data": [
            {
                "forwardCurves": [{"curvePoints": []}],
            }
        ]
    }
)

FORWARD_CURVES_RESPONSE = FORWARD_CURVE_RESPONSE

ZC_CURVE_RESPONSE = StubResponse(
    {
        "data": [
            {
                "curves": {"key": {"curvePoints": []}},
            }
        ]
    }
)

ZC_CURVE_DEFINITION_RESPONSE = StubResponse(
    {
        "data": [
            {
                "curveDefinitions": [{}],
            }
        ]
    }
)

ZC_CURVE_DEFINITIONS_RESPONSE = ZC_CURVE_DEFINITION_RESPONSE

ZC_CURVES_RESPONSE = StubResponse(
    {
        "data": [
            {
                "curves": {"key": {"curvePoints": []}},
            }
        ]
    }
)

SURFACE_CAP_DEFINITION = rdc.ipa.surfaces.cap.Definition(
    surface_tag="USD_Strike__Tenor_",
    underlying_definition=cap.CapSurfaceDefinition(
        instrument_code="USD",
        discounting_type=cap.DiscountingType.OIS_DISCOUNTING,
    ),
    surface_layout=cap.SurfaceLayout(format=cap.Format.MATRIX),
    surface_parameters=cap.CapCalculationParams(
        valuation_date="2020-03-20",
        x_axis=cap.Axis.STRIKE,
        y_axis=cap.Axis.TENOR,
    ),
)

SURFACE_CAP_RESPONSE = StubResponse(
    {
        "data": [
            {
                "surfaceTag": "USD_Strike__Tenor_",
                "surface": [
                    [
                        None,
                        "0.250000",
                    ],
                    [
                        "3",
                        "49.20",
                    ],
                    [
                        "9",
                        "49.20",
                    ],
                ],
            }
        ]
    }
)

SURFACE_ETI_DEFINITION = rdc.ipa.surfaces.eti.Definition(
    surface_tag="1",
    underlying_definition=rdc.ipa.surfaces.eti.EtiSurfaceDefinition(
        instrument_code="BNPP.PA@RIC",
    ),
    surface_parameters=rdc.ipa.surfaces.eti.EtiCalculationParams(
        price_side=rdc.ipa.surfaces.eti.PriceSide.MID,
        volatility_model=rdc.ipa.surfaces.eti.VolatilityModel.SVI,
        x_axis=rdc.ipa.surfaces.cap.Axis.DATE,
        y_axis=rdc.ipa.surfaces.cap.Axis.STRIKE,
    ),
    surface_layout=rdc.ipa.surfaces.eti.SurfaceLayout(
        format=rdc.ipa.surfaces.cap.Format.MATRIX, y_point_count=10
    ),
)

SURFACE_ETI_RESPONSE = StubResponse(
    {
        "data": [
            {
                "surfaceTag": "1",
                "surface": [
                    [
                        None,
                        "37.656",
                        "40.0095",
                        "42.363",
                    ],
                    [
                        "2022-04-14",
                        100.646188439577,
                        85.3825756996221,
                        71.1129204322784,
                    ],
                    [
                        "2022-05-20",
                        63.6892177011107,
                        58.93280211148571,
                        54.4441693428238,
                    ],
                ],
            }
        ]
    }
)

SURFACE_FX_DEFINITION = rdc.ipa.surfaces.fx.Definition(
    underlying_definition={"fxCrossCode": "EURUSD"},
    surface_tag="FxVol-EURUSD",
    surface_layout=fx.SurfaceLayout(format=fx.Format.MATRIX),
    surface_parameters=fx.FxCalculationParams(
        x_axis=fx.Axis.DATE,
        y_axis=fx.Axis.STRIKE,
        calculation_date="2018-08-20T00:00:00Z",
    ),
)

SURFACE_FX_RESPONSE = StubResponse(
    {
        "data": [
            {
                "surfaceTag": "FxVol-EURUSD",
                "surface": [
                    [
                        None,
                        1.1038366375817679,
                        1.109416864898416,
                        1.114997092215064,
                    ],
                    [
                        "2018-08-21T00:00:00Z",
                        14.680556199566963,
                        13.940204590775027,
                        13.164852627183423,
                    ],
                    [
                        "2018-08-27T00:00:00Z",
                        9.405878307234056,
                        9.105090369857033,
                        8.797664558348064,
                    ],
                ],
            }
        ]
    }
)

NONE_REPLACED_WITH_NA_RDP_DATA = {
    "data": [
        ["GOOG.O", "2020-01-20T00:00:00Z", None],
        ["GOOG.O", "2020-12-31T00:00:00Z", None],
    ],
    "headers": [
        {"name": "instrument", "title": "Instrument"},
        {"name": "date", "title": "Date"},
        {"name": "TR.RevenueMean", "title": "Currency"},
    ],
}

NONE_REPLACED_WITH_NA_UDF_DATA = {
    "data": [
        ["GOOG.O", "2020-01-20T00:00:00Z", None],
        ["GOOG.O", "2020-12-31T00:00:00Z", None],
    ],
    "headers": [
        [
            {"displayName": "Instrument"},
            {"displayName": "Date"},
            {"displayName": "Currency"},
        ]
    ],
}

NAN_REPLACED_WITH_NA_RDP_DATA = {
    "data": [
        ["GOOG.O", "2020-01-20T00:00:00Z", 1000],
        ["GOOG.O", "2020-12-31T00:00:00Z", None],
    ],
    "headers": [
        {"name": "instrument", "title": "Instrument"},
        {"name": "date", "title": "Date"},
        {"name": "TR.RevenueMean", "title": "Currency"},
    ],
}

NAN_REPLACED_WITH_NA_UDF_DATA = {
    "data": [
        ["GOOG.O", "2020-01-20T00:00:00Z", 1000],
        ["GOOG.O", "2020-12-31T00:00:00Z", None],
    ],
    "headers": [
        [
            {"displayName": "Instrument"},
            {"displayName": "Date"},
            {"displayName": "Currency"},
        ]
    ],
}

EMPTY_STRING_LEAVE_AS_IT_IS_RDP_DATA = {
    "data": [
        ["GOOG.O", "2020-01-20T00:00:00Z", ""],
        ["GOOG.O", "2020-12-31T00:00:00Z", ""],
    ],
    "headers": [
        {"name": "instrument", "title": "Instrument"},
        {"name": "date", "title": "Date"},
        {"name": "TR.RevenueMean", "title": "Currency"},
    ],
}
EMPTY_STRING_LEAVE_AS_IT_IS_UDF_DATA = {
    "data": [
        ["GOOG.O", "2020-01-20T00:00:00Z", ""],
        ["GOOG.O", "2020-12-31T00:00:00Z", ""],
    ],
    "headers": [
        [
            {"displayName": "Instrument"},
            {"displayName": "Date"},
            {"displayName": "Currency"},
        ]
    ],
}

TWO_UNIVERSES_ONE_FIELD_RDP_DATA = {
    "data": [
        ["EUR=", "2022-01-31", 2000],
        ["EUR=", "2022-02-28", 2000],
        ["LSEG.L", "2020-01-20T00:00:00Z", 1000],
        ["LSEG.L", "2020-12-31T00:00:00Z", 1000],
    ],
    "headers": [
        {"name": "instrument", "title": "Instrument"},
        {"name": "date", "title": "Date"},
        {"name": "BID", "title": "BID"},
    ],
}
TWO_UNIVERSES_ONE_FIELD_UDF_DATA = {
    "data": [
        ["EUR=", "2022-01-31", 2000],
        ["EUR=", "2022-02-28", 2000],
        ["LSEG.L", "2020-01-20T00:00:00Z", 1000],
        ["LSEG.L", "2020-12-31T00:00:00Z", 1000],
    ],
    "headers": [
        [
            {"displayName": "Instrument"},
            {"displayName": "Date"},
            {"displayName": "BID"},
        ]
    ],
}

INSTRUMENT_AND_CURRENCY_ARE_EMPTY_STRINGS_RDP_DATA = {
    "data": [
        ["", "2020-01-20T00:00:00Z", ""],
        ["GOOG.O", "2020-12-31T00:00:00Z", 1000],
    ],
    "headers": [
        {"name": "instrument", "title": "Instrument"},
        {"name": "date", "title": "Date"},
        {"name": "TR.RevenueMean", "title": "Currency"},
    ],
}
INSTRUMENT_AND_CURRENCY_ARE_EMPTY_STRINGS_UDF_DATA = {
    "data": [
        ["", "2020-01-20T00:00:00Z", ""],
        ["GOOG.O", "2020-12-31T00:00:00Z", 1000],
    ],
    "headers": [
        [
            {"displayName": "Instrument"},
            {"displayName": "Date"},
            {"displayName": "Currency"},
        ]
    ],
}

headers_rdp = [
    {"name": "instrument", "title": "Instrument"},
    {"name": "date", "title": "Date"},
    {"name": "TR.RevenueMean", "title": "Currency"},
    {"name": "TR.RevenueMean", "title": "Revenue - Mean"},
]

headers_udf = [
    [
        {"displayName": "Instrument"},
        {"displayName": "Date"},
        {"displayName": "Currency", "field": "TR.REVENUEMEAN.currency"},
        {"displayName": "Revenue - Mean", "field": "TR.REVENUEMEAN"},
    ]
]

TWO_SAME_DATES_DOES_NOT_MERGE_IN_ONE_RDP_DATA = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["IBM", "2020-12-16T00:00:00", "USD", 73950729070],
        ["IBM", "2021-01-27T00:00:00", "USD", 74195199870],
        ["IBM", "2021-02-11T00:00:00", "USD", 74195199870],
        ["IBM", "2021-03-22T00:00:00", "USD", 74221312380],
        ["IBM", "2021-04-25T00:00:00", "USD", 74397055190],
        ["IBM", "2021-05-21T00:00:00", "USD", 74329699000],
        ["IBM", "2021-05-21T00:00:00", "USD", 74329699000],
        ["IBM", "2021-07-29T00:00:00", "USD", 75129376130],
        ["IBM", "2021-08-30T00:00:00", "USD", 75127177530],
    ],
    "headers": headers_rdp,
}
TWO_SAME_DATES_DOES_NOT_MERGE_IN_ONE_UDF_DATA = {
    "data": [
        ["IBM", "2020-10-23T00:00:00Z", "USD", 73965242400],
        ["IBM", "2020-10-23T00:00:00Z", "USD", 73965242400],
        ["IBM", "2020-12-16T00:00:00Z", "USD", 73950729070],
        ["IBM", "2021-01-27T00:00:00Z", "USD", 74195199870],
        ["IBM", "2021-02-11T00:00:00Z", "USD", 74195199870],
        ["IBM", "2021-03-22T00:00:00Z", "USD", 74221312380],
        ["IBM", "2021-04-25T00:00:00Z", "USD", 74397055190],
        ["IBM", "2021-05-21T00:00:00Z", "USD", 74329699000],
        ["IBM", "2021-05-21T00:00:00Z", "USD", 74329699000],
        ["IBM", "2021-07-29T00:00:00Z", "USD", 75129376130],
        ["IBM", "2021-08-30T00:00:00Z", "USD", 75127177530],
    ],
    "headers": headers_udf,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_RDP_DATA = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["IBM", "2020-12-16T00:00:00", "USD", 73950729070],
        ["IBM", "2021-01-27T00:00:00", "USD", 74195199870],
        ["IBM", "2021-02-11T00:00:00", "USD", 74195199870],
        ["IBM", "2021-03-22T00:00:00", "USD", 74221312380],
        ["IBM", "2021-04-25T00:00:00", "USD", 74397055190],
        ["IBM", "2021-05-21T00:00:00", "USD", 74329699000],
        ["IBM", "2021-05-21T00:00:00", "USD", 74329699000],
        ["IBM", "2021-07-29T00:00:00", "USD", 75129376130],
        ["IBM", "2021-08-30T00:00:00", "USD", 75127177530],
        ["VOD.L", "2020-10-21T00:00:00", "EUR", 43375027760],
        ["VOD.L", "2020-11-23T00:00:00", "EUR", 43442990380],
        ["VOD.L", "2020-12-16T00:00:00", "EUR", 43372965910],
        ["VOD.L", "2021-01-20T00:00:00", "EUR", 43333798760],
        ["VOD.L", "2021-02-24T00:00:00", "EUR", 43421860050],
        ["VOD.L", "2021-03-31T00:00:00", "EUR", 43445947190],
        ["VOD.L", "2021-04-28T00:00:00", "EUR", 43503348730],
        ["VOD.L", "2021-05-27T00:00:00", "EUR", 44918585000],
        ["VOD.L", "2021-06-29T00:00:00", "EUR", 44967283500],
        ["VOD.L", "2021-07-29T00:00:00", "EUR", 45109776750],
        ["VOD.L", "2021-08-25T00:00:00", "EUR", 45088444860],
    ],
    "headers": headers_rdp,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_UDF_DATA = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["IBM", "2020-12-16T00:00:00", "USD", 73950729070],
        ["IBM", "2021-01-27T00:00:00", "USD", 74195199870],
        ["IBM", "2021-02-11T00:00:00", "USD", 74195199870],
        ["IBM", "2021-03-22T00:00:00", "USD", 74221312380],
        ["IBM", "2021-04-25T00:00:00", "USD", 74397055190],
        ["IBM", "2021-05-21T00:00:00", "USD", 74329699000],
        ["IBM", "2021-05-21T00:00:00", "USD", 74329699000],
        ["IBM", "2021-07-29T00:00:00", "USD", 75129376130],
        ["IBM", "2021-08-30T00:00:00", "USD", 75127177530],
        ["VOD.L", "2020-10-21T00:00:00", "EUR", 43375027760],
        ["VOD.L", "2020-11-23T00:00:00", "EUR", 43442990380],
        ["VOD.L", "2020-12-16T00:00:00", "EUR", 43372965910],
        ["VOD.L", "2021-01-20T00:00:00", "EUR", 43333798760],
        ["VOD.L", "2021-02-24T00:00:00", "EUR", 43421860050],
        ["VOD.L", "2021-03-31T00:00:00", "EUR", 43445947190],
        ["VOD.L", "2021-04-28T00:00:00", "EUR", 43503348730],
        ["VOD.L", "2021-05-27T00:00:00", "EUR", 44918585000],
        ["VOD.L", "2021-06-29T00:00:00", "EUR", 44967283500],
        ["VOD.L", "2021-07-29T00:00:00", "EUR", 45109776750],
        ["VOD.L", "2021-08-25T00:00:00", "EUR", 45088444860],
    ],
    "headers": headers_udf,
}

# Combinations:
# 1. x x -
# 2. x x x
# 3. - - -
# 4. - x x
# 5. x - x

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_RDP_DATA_1 = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-05-07T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_rdp,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_UDF_DATA_1 = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-05-07T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_udf,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_RDP_DATA_2 = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-10-23T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_rdp,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_UDF_DATA_2 = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-10-23T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_udf,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_RDP_DATA_3 = {
    "data": [
        ["IBM", "2020-09-18T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-05-07T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_rdp,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_UDF_DATA_3 = {
    "data": [
        ["IBM", "2020-09-18T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-05-07T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_udf,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_RDP_DATA_4 = {
    "data": [
        ["IBM", "2020-05-07T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-10-23T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_rdp,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_UDF_DATA_4 = {
    "data": [
        ["IBM", "2020-05-07T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-10-23T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-10-23T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_udf,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_RDP_DATA_5 = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-05-07T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-10-23T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_rdp,
}

MERGE_VALUES_WITH_SAME_DATE_IN_ONE_ROW_UDF_DATA_5 = {
    "data": [
        ["IBM", "2020-10-23T00:00:00", "USD", 73965242400],
        ["VOD.L", "2020-05-07T00:00:00", "EUR", 43375027760],
        ["NKE.N", "2020-10-23T00:00:00", "GBP", 43375027760],
    ],
    "headers": headers_udf,
}

DELETE_ROW_IF_DATE_IS_NONE_UDF_DATA = {
    "data": [
        ["GOOG.O", "", "", "", None],
        ["GOOG.O", "2021-12-31T00:00:00Z", 257637000000, 146698000000, ""],
        ["INVAL", "", None, None, None],
        ["FB.O", "", "", "", None],
        ["FB.O", "2020-12-31T00:00:00Z", 85965000000, 69273000000, ""],
    ],
    "headers": [
        [
            {"displayName": "Instrument"},
            {"displayName": "Date"},
            {"displayName": "Revenue", "field": "TR.REVENUE"},
            {"displayName": "Gross Profit", "field": "TR.GROSSPROFIT"},
            {"displayName": "INVALID_FIELD", "field": "INVALID_FIELD"},
        ]
    ],
}
DELETE_ROW_IF_DATE_IS_NONE_RDP_DATA = {
    "data": [
        ["GOOG.O", "2021-12-31T00:00:00", 257637000000, 146698000000],
        ["INVAL", None, None, None],
        ["FB.O", "2020-12-31T00:00:00", 85965000000, 69273000000],
    ],
    "headers": [
        {"name": "instrument", "title": "Instrument"},
        {"name": "date", "title": "Date"},
        {"name": "TR.Revenue", "title": "Revenue"},
        {"name": "TR.GrossProfit", "title": "Gross Profit"},
    ],
}
zc_curves_definitions = [
    rdc.ipa.curves.zc_curve_definitions.Definition(source="Refinitiv"),
    rdc.ipa.curves.zc_curve_definitions.Definition(source="Peugeot"),
]
calendars_definitions = [
    rdc.ipa.dates_and_calendars.add_periods.Definition(
        tag="first",
        start_date="2020-01-01",
    ),
    rdc.ipa.dates_and_calendars.add_periods.Definition(
        tag="second",
        start_date="2018-01-01",
    ),
]
universe_definitions = [
    rdc.esg.basic_overview.Definition,
    rdc.esg.full_scores.Definition,
    rdc.esg.full_measures.Definition,
    rdc.esg.standard_scores.Definition,
    rdc.esg.standard_measures.Definition,
    partial(rdc.estimates.view_actuals.annual.Definition, package="basic"),
    partial(rdc.estimates.view_actuals.interim.Definition, package="basic"),
    rdc.estimates.view_actuals_kpi.annual.Definition,
    rdc.estimates.view_actuals_kpi.interim.Definition,
    partial(rdc.estimates.view_summary.annual.Definition, package="basic"),
    partial(
        rdc.estimates.view_summary.historical_snapshots_non_periodic_measures.Definition,
        package="basic",
    ),
    partial(
        rdc.estimates.view_summary.historical_snapshots_periodic_measures_annual.Definition,
        package="basic",
    ),
    partial(
        rdc.estimates.view_summary.historical_snapshots_periodic_measures_interim.Definition,
        package="basic",
    ),
    partial(
        rdc.estimates.view_summary.historical_snapshots_recommendations.Definition,
        package="basic",
    ),
    partial(rdc.estimates.view_summary.interim.Definition, package="basic"),
    partial(
        rdc.estimates.view_summary.non_periodic_measures.Definition, package="basic"
    ),
    partial(rdc.estimates.view_summary.recommendations.Definition, package="basic"),
    rdc.estimates.view_summary_kpi.annual.Definition,
    rdc.estimates.view_summary_kpi.historical_snapshots_kpi.Definition,
    rdc.estimates.view_summary_kpi.interim.Definition,
    partial(rdc.ownership.consolidated.breakdown.Definition, stat_type=1),
    rdc.ownership.consolidated.concentration.Definition,
    rdc.ownership.consolidated.investors.Definition,
    partial(rdc.ownership.consolidated.recent_activity.Definition, sort_order="asc"),
    rdc.ownership.consolidated.shareholders_report.Definition,
    partial(rdc.ownership.consolidated.top_n_concentration.Definition, count=5),
    partial(rdc.ownership.fund.breakdown.Definition, stat_type=1),
    rdc.ownership.fund.concentration.Definition,
    rdc.ownership.fund.holdings.Definition,
    rdc.ownership.fund.investors.Definition,
    partial(rdc.ownership.fund.recent_activity.Definition, sort_order="asc"),
    rdc.ownership.fund.shareholders_report.Definition,
    partial(rdc.ownership.fund.top_n_concentration.Definition, count=5),
    rdc.ownership.insider.shareholders_report.Definition,
    rdc.ownership.insider.transaction_report.Definition,
    rdc.ownership.investor.holdings.Definition,
    rdc.ownership.org_info.Definition,
    rdc.pricing.Definition,
]
fields_in_json_definitions = [
    rdc.ipa.financial_contracts.bond.Definition,
    rdc.ipa.financial_contracts.cap_floor.Definition,
    rdc.ipa.financial_contracts.cds.Definition,
    rdc.ipa.financial_contracts.cross.Definition,
    partial(
        rdc.ipa.financial_contracts.Definitions,
        universe=[rdc.ipa.financial_contracts.option.Definition()],
    ),
    rdc.ipa.financial_contracts.option.Definition,
    rdc.ipa.financial_contracts.repo.Definition,
    rdc.ipa.financial_contracts.swap.Definition,
    rdc.ipa.financial_contracts.swaption.Definition,
    rdc.ipa.financial_contracts.term_deposit.Definition,
]
calendars_jsons = [
    {"startDate": "2020-01-01", "tag": "first", "holidayOutputs": []},
    {"startDate": "2018-01-01", "tag": "second", "holidayOutputs": []},
]



top_news_headlines_df = """                                                                                                   headline                                                                                                                                                                                                                                                                                        snippet                                     storyId                                               imageId
versionCreated                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
2022-11-22 11:56:00                                       Europe to be hit hardest in global slowdown -OECD                                                  The global economy should avoid a recession next year but the worst energy crisis since the 1970s will trigger a sharp slowdown, with Europe hit hardest, the OECD said, adding that fighting inflation should be policymakers' top priority.  urn:newsml:reuters.com:20221122:nL1N32I0MO  2022-11-22T100711Z_1_AT0_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:59:00           EXCLUSIVE-Bankman-Fried's FTX, parents bought Bahamas property worth $121 mln                                                               Sam Bankman-Fried's FTX, his parents and senior executives of the failed cryptocurrency exchange bought at least 19 properties worth nearly $121 million in the Bahamas over the past two years, official property records show.  urn:newsml:reuters.com:20221122:nL1N32I0A4  2022-11-22T061143Z_1_NT4_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:31:00  EXCLUSIVE-China set to fine Ant Group over $1 bln, signalling revamp nears end-sources                                                    Chinese authorities are poised to impose a fine of more than $1 billion on Jack Ma's Ant Group, said six sources with direct knowledge of the matter, setting the stage for ending the fintech company's two-year long regulatory overhaul.  urn:newsml:reuters.com:20221122:nL1N32I0MJ  2022-11-22T090637Z_1_SY6_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:21:00             Ukrainians brace for bleak winter as Russian strikes cripple power capacity                                    President Volodymyr Zelenskiy appealed to Ukrainians to conserve energy amid relentless Russian strikes that have already halved the country's power capacity, as the United Nations' health body warned of a humanitarian disaster in Ukraine this winter.  urn:newsml:reuters.com:20221121:nL1N32H2DO  2022-11-22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 12:07:00                      GLOBAL MARKETS-Shares brush off China COVID curbs, dollar retreats                                                           Global shares rose on Tuesday, recovering some of the previous day's losses, as improved investor risk appetite drove flows into equities and commodities, although concern over more COVID infections in China kept gains in check.  urn:newsml:reuters.com:20221122:nL1N32I0QM  2022-11-22T085147Z_1_LW3_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:06:00                                Beijing shuts parks, museums as China's COVID cases rise                                                 Beijing shut parks, malls and museums on Tuesday while more Chinese cities resumed mass testing for COVID-19 as authorities struggle with a spike in cases that has deepened concern about the economy and dimmed hopes for a quick reopening.  urn:newsml:reuters.com:20221122:nL1N32I03J  2022-11-22T112536Z_1_FB1_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:09:00                                                 Cryptoverse: Let's talk about DEX, baby                                                                                                                                                                  As the crypto castle crumbles, some true believers say the answer is to double down on DEX. Decentralized exchanges, that is.  urn:newsml:reuters.com:20221122:nL4N32E3CU  2022-11-22T062251Z_1_GY2_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:59:00           ANALYSIS-Kohl's snub of big sale-leaseback sets up new clash with hedge funds                                                                                                                        Kohl's Corp's reluctance to sell much of its real estate and then lease it back has set up the U.S. department store operator for a new confrontation with hedge funds.  urn:newsml:reuters.com:20221122:nL1N32B2UJ  2022-11-22T111327Z_1_JZ3_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:32:00                           Don't like Musk? Work for us! Tech firms woo ex-Twitter staff                                                Put off by Elon Musk's muscular management style? Move to us! That's the pitch being used by talent-starved technology firms trying to lure thousands of former Twitter Inc employees laid off by the social media company under its new owner.  urn:newsml:reuters.com:20221122:nL8N32H3XG  2022-11-22T104018Z_1_RH6_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 09:43:00        ANALYSIS-China has limited power, and perhaps little desire, to curb North Korea                                                                U.S. President Joe Biden last week asked his Chinese counterpart, Xi Jinping, if he could try to talk North Korea out of a new nuclear weapon test but he acknowledged that he did not know if China had the capacity to do so.  urn:newsml:reuters.com:20221122:nL4N32F06Z  2022-11-22T095230Z_1_QY5_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 09:45:00               EU gas cap seen in place for one year, countries at odds over price level                                                                                                The European Commission proposed introducing a gas price cap for one year from Jan.1, 2023, according to draft legislation seen by Reuters that has so far left the actual ceiling level blank.  urn:newsml:reuters.com:20221122:nL8N32I2CE  2022-11-22T092043Z_1_WT7_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 10:59:00                 FOCUS-Generic drugmakers Teva and Sandoz make major push to biosimilars  Generic drug makers Teva Pharmaceutical Industries and Sandoz say they are planning a significant ramp-up in production of biosimilars – copies of high-priced drugs used to treat illnesses such as rheumatoid arthritis and cancer – aiming to increase their share of an expanding market.  urn:newsml:reuters.com:20221122:nL6N32D0C3  2022-11-22T110845Z_1_FN2_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 05:29:00               'Boomerang CEOs' don't always work out; Disney hopes this one bucks trend                                                                                                          Can Walt Disney Co bank on another hit sequel? That appears to be the hope behind the company's surprise decision to bring back former chief executive BobIger to replace Bob Chapek.  urn:newsml:reuters.com:20221122:nL1N32H1O7  2022-11-22T111759Z_1_YH8_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 11:38:00                            Many school children among 252 dead in Indonesian earthquake                                                                                      Many children were killed in an earthquake in Indonesia when their schools collapsed, officials said on Tuesday, as rescuers raced to reach people trapped in rubble in a devastated town on Java island.  urn:newsml:reuters.com:20221122:nL1N32I01R  2022-11-22T093118Z_1_UY6_RTRLXPP_2_LYNXPACKAGER__JPG
2022-11-22 09:21:00          China's COVID resurgence delays oil demand recovery to after Q2 2023 -analysts                                                                     Analysts are cutting forecasts for China's year-end oil demand after COVID-19 cases surged to near record levels, forcing authorities to reinstate mobility curbs and delaying recovery at the world's top crude importer.  urn:newsml:reuters.com:20221122:nL1N32I0IM  2022-11-22T024400Z_1_LM3_RTRLXPP_2_LYNXPACKAGER__JPG"""



from refinitiv.data.content._df_builder import build_empty_df
from refinitiv.data.content.search._data_provider import SearchData
from refinitiv.data.content import search

MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE = {
    "Total": 606753,
    "Hits": [
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "ICE-US FCOJ-A Futures Electronic Commodity Future Continuation 1, Commodity Future, Intercontinental Exchange US",
            "PermID": "21480992515",
            "PI": "16866",
            "RIC": "OJc1",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "ICE-US FCOJ-A Futures Electronic Commodity Future Chain Chain Contracts, Commodity Future, Intercontinental Exchange US",
            "PermID": "21481145637",
            "PI": "73603165",
            "RIC": "0#OJ:",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "ICE-US FCOJ-A Futures Electronic Commodity Future Continuation 2, Commodity Future, Intercontinental Exchange US",
            "PermID": "21480996404",
            "PI": "1095831",
            "RIC": "OJc2",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "CFTC Future and Option Commitments FRZN Concentrated Orange Juice - ICE Futures U.S. Contract Total Open Interest Contracts of 15000 Pounds 040701, Energy Statistic, Commodity Futures Trading Commission",
            "PermID": "21481534494",
            "PI": "7956248",
            "RIC": "3CFTC040701OI",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "CFTC Future and Option Commitments FRZN Concentrated Orange Juice - ICE Futures U.S. Managed Net Contracts of 15000 Pounds 040701, Energy Statistic, Commodity Futures Trading Commission",
            "PermID": "21478081931",
            "PI": "63261287",
            "RIC": "3040701MNET",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "CFTC Future and Option Commitments FRZN Concentrated Orange Juice - ICE Futures U.S. Producer Net Contracts of 15000 Pounds 040701, Energy Statistic, Commodity Futures Trading Commission",
            "PermID": "21478080548",
            "PI": "63259739",
            "RIC": "3040701PNET",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "CFTC Future and Option Commitments FRZN Concentrated Orange Juice - ICE Futures U.S. Swap Net Contracts of 15000 Pounds 040701, Energy Statistic, Commodity Futures Trading Commission",
            "PermID": "21478081878",
            "PI": "63261232",
            "RIC": "3040701SNET",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "CFTC Future and Option Commitments FRZN Concentrated Orange Juice - ICE Futures U.S. Other Net Contracts of 15000 Pounds 040701, Energy Statistic, Commodity Futures Trading Commission",
            "PermID": "21478082733",
            "PI": "63262308",
            "RIC": "3040701ONET",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "ICE-US FCOJ-A Futures Electronic Commodity Future Jan 2023, Commodity Future, Intercontinental Exchange US",
            "PermID": "21732946183",
            "PI": "413283469",
            "RIC": "OJF3",
        },
        {
            "BusinessEntity": "QUOTExCOMMODITY",
            "DocumentTitle": "ICE-US FCOJ-A Futures Electronic Commodity Future Mar 2023, Commodity Future, Intercontinental Exchange US",
            "PermID": "21744573309",
            "PI": "427951324",
            "RIC": "OJH3",
        },
    ],
    "Navigators": {
        "MaturityDate": {
            "Buckets": [
                {
                    "Label": "2020-01-01",
                    "Filter": "(MaturityDate ge 2020-01-01 and MaturityDate lt 2021-01-01)",
                    "Count": 408,
                    "BusinessEntity": {
                        "Buckets": [
                            {
                                "Label": "INSTRUMENTxFIXEDINCOMExGOVCORP",
                                "Count": 408,
                                "sum_FaceOutstandingUSD": 48138904788.0,
                            }
                        ]
                    },
                },
                {
                    "Label": "2021-01-01",
                    "Filter": "(MaturityDate ge 2021-01-01 and MaturityDate lt 2022-01-01)",
                    "Count": 561,
                    "BusinessEntity": {
                        "Buckets": [
                            {
                                "Label": "INSTRUMENTxFIXEDINCOMExGOVCORP",
                                "Count": 561,
                                "sum_FaceOutstandingUSD": 53567730098.0,
                            }
                        ]
                    },
                },
                {
                    "Label": "2022-01-01",
                    "Filter": "(MaturityDate ge 2022-01-01 and MaturityDate lt 2023-01-01)",
                    "Count": 73963,
                    "BusinessEntity": {
                        "Buckets": [
                            {
                                "Label": "INSTRUMENTxFIXEDINCOMExGOVCORP",
                                "Count": 73963,
                                "sum_FaceOutstandingUSD": 1523293417439.9312,
                            }
                        ]
                    },
                },
                {
                    "Label": "2023-01-01",
                    "Filter": "(MaturityDate ge 2023-01-01 and MaturityDate lt 2024-01-01)",
                    "Count": 309738,
                    "BusinessEntity": {
                        "Buckets": [
                            {
                                "Label": "INSTRUMENTxFIXEDINCOMExGOVCORP",
                                "Count": 309738,
                                "sum_FaceOutstandingUSD": 8394626310316.518,
                            }
                        ]
                    },
                },
                {
                    "Label": "2024-01-01",
                    "Filter": "(MaturityDate ge 2024-01-01 and MaturityDate lt 2025-01-01)",
                    "Count": 132494,
                    "BusinessEntity": {
                        "Buckets": [
                            {
                                "Label": "INSTRUMENTxFIXEDINCOMExGOVCORP",
                                "Count": 132494,
                                "sum_FaceOutstandingUSD": 5237852833418.076,
                            }
                        ]
                    },
                },
                {
                    "Label": "2025-01-01",
                    "Filter": "MaturityDate ge 2025-01-01",
                    "Count": 89589,
                    "BusinessEntity": {
                        "Buckets": [
                            {
                                "Label": "INSTRUMENTxFIXEDINCOMExGOVCORP",
                                "Count": 89589,
                                "sum_FaceOutstandingUSD": 5333550586785.237,
                            }
                        ]
                    },
                },
            ]
        }
    },
}


def test_workspace():
    assert hasattr(search, "Definition")


def test_attributes():
    # given
    excepted_attributes = [
        "_query",
        "_view",
        "_filter",
        "_order_by",
        "_boost",
        "_select",
        "_top",
        "_skip",
        "_group_by",
        "_group_count",
        "_navigators",
        "_features",
        "_scope",
        "_terms",
        "_extended_params",
        #
        "_kwargs",
        "_data_type",
        "_content_type",
        "_provider",
    ]

    # when
    definition = search.Definition()
    attributes = list(definition.__dict__.keys())

    # then
    assert len(attributes) == len(excepted_attributes)
    assert set(attributes) == set(excepted_attributes)


def test_definition_search_repr():
    # given
    definition = search.Definition()
    obj_id = hex(id(definition))
    expected_value = f"<refinitiv.data.content.search.Definition object at {obj_id} {{query='None'}}>"

    # when
    testing_value = repr(definition)

    # then
    assert testing_value == expected_value


def test_search_navigators():
    response = SearchData(MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE, build_empty_df)

    assert (
        response.hits[0].DocumentTitle
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Hits"][0]["DocumentTitle"]
    )
    assert (
        response.hits[0].BusinessEntity
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Hits"][0]["BusinessEntity"]
    )
    assert (
        response.hits[0].PermID
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Hits"][0]["PermID"]
    )
    assert response.hits[0].PI == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Hits"][0]["PI"]
    assert (
        response.hits[0].RIC == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Hits"][0]["RIC"]
    )
    assert (
        response.navigators["MaturityDate"].buckets[0].label
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Navigators"]["MaturityDate"][
            "Buckets"
        ][0]["Label"]
    )
    assert (
        response.navigators["MaturityDate"].buckets[0].count
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Navigators"]["MaturityDate"][
            "Buckets"
        ][0]["Count"]
    )
    assert response.navigators["MaturityDate"].buckets[0].navigator.name
    assert response.navigators["MaturityDate"].name == "MaturityDate"
    assert (
        response.navigators["MaturityDate"].buckets[0].navigator.buckets[0].label
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Navigators"]["MaturityDate"][
            "Buckets"
        ][0]["BusinessEntity"]["Buckets"][0]["Label"]
    )
    assert (
        response.navigators["MaturityDate"].buckets[0]["Filter"]
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Navigators"]["MaturityDate"][
            "Buckets"
        ][0]["Filter"]
    )
    assert (
        response.navigators["MaturityDate"]
        .buckets[0]
        .navigator.buckets[0]["sum_FaceOutstandingUSD"]
        == MOCKED_SEARCH_NAVIGATORS_RAW_RESPONSE["Navigators"]["MaturityDate"][
            "Buckets"
        ][0]["BusinessEntity"]["Buckets"][0]["sum_FaceOutstandingUSD"]
    )



from typing import TYPE_CHECKING

from ._data_provider import HierarchyData
from ....._content_type import ContentType
from .....delivery._data._data_provider_layer import DataProviderLayer
from .....delivery._data._response import BaseResponse

if TYPE_CHECKING:
    from typing import Optional, Union
    from ._top_news_id import TopNewsId


class Definition(DataProviderLayer[BaseResponse[HierarchyData]]):
    """
    This class describes parameters to retrieve data for top news hierarchy.

    Parameters
    ----------
    id: str or Enum, optional
        The id of the package[current, test, next]

     Examples
    --------
    >>> from refinitiv.data.content import news
    >>> definition = news.top_news.hierarchy.Definition()
    >>> response = definition.get_data()
    >>> response.data.hierarchy['Commodities']
    """

    def __init__(
        self,
        id: "Optional[Union[TopNewsId, str]]" = None,
    ):
        super().__init__(data_type=ContentType.NEWS_TOP_NEWS, id=id)



from typing import Optional, TYPE_CHECKING

from ..._content_data import Data
from ..._content_provider_layer import ContentUsageLoggerMixin
from ...._content_type import ContentType
from ...._tools import validate_types, validate_bool_value, try_copy_to_list
from ....delivery._data._data_provider import DataProviderLayer, BaseResponse

if TYPE_CHECKING:
    from ...._types import ExtendedParams, OptDateTime, StrStrings


class Definition(
    ContentUsageLoggerMixin[BaseResponse[Data]],
    DataProviderLayer[BaseResponse[Data]],
):
    """
    This class describe parameters to retrieve details on stakeholders and strategic entities
    transactions that purchased the requested instruments. Further details of insider stakeholder
    can be requested along with their holding details. The operation supports pagination, however,
    it is dependent on user entitlements. Maximum 'count' value per page is 100. The default date
    range is 20 transactions, unless the 'start date' and 'end date' define a smaller range. The
    count value is checked by the service to determine if it does not exceed a specific number. If
    it does, the service will overwrite the client value to service default value.

    Parameters
    ----------
    universe: str, list of str
        The Universe parameter allows the user to define the single company for which the content is returned.

    start: str, datetime, optional
        The start parameter allows users to define the start date of a time series.
        Dates are to be defined either by absolute or relative syntax.
        Example, 20190529, -1Q, 1D, -3MA.

    end: str, datetime, optional
        The end parameter allows users to define the start date of a time series.
        Dates are to be defined either by absolute or relative syntax.
        Example, 20190529, -1Q, 1D, -3MA.

    limit: int, optional
        The limit parameter is used for paging. It allows users to select the number of records to be returned.

    use_field_names_in_headers: bool, optional
        Return field name as column headers for data instead of title

    extended_params : ExtendedParams, optional
        If necessary other parameters.

    Examples
    --------
    >>> from refinitiv.data.content import ownership
    >>> definition = ownership.insider.transaction_report.Definition("TRI.N", start="-1Q")
    >>> response = definition.get_data()
    """

    _USAGE_CLS_NAME = "Ownership.Insider.TransactionReportDefinition"

    def __init__(
        self,
        universe: "StrStrings",
        start: "OptDateTime" = None,
        end: "OptDateTime" = None,
        limit: Optional[int] = None,
        use_field_names_in_headers: bool = False,
        extended_params: "ExtendedParams" = None,
    ):
        validate_types(limit, [int, type(None)], "limit")
        validate_bool_value(use_field_names_in_headers)
        universe = try_copy_to_list(universe)

        super().__init__(
            ContentType.OWNERSHIP_INSIDER_TRANSACTION_REPORT,
            universe=universe,
            start=start,
            end=end,
            limit=limit,
            use_field_names_in_headers=use_field_names_in_headers,
            extended_params=extended_params,
        )



import threading
import time
from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Callable, Set, Union
from ._scope_map import SCOPE_BY_KEY
from .access_token_updater import AccessTokenUpdater
from .event import UpdateEvent
from .refresh_token_updater import RefreshTokenUpdater
from .tools import MINUTES_10, get_delays
from ..log_reporter import LogReporter
from ..._errors import ScopeError
from ..._tools import cached_property, CallbackHandler

if TYPE_CHECKING:
    from ._platform_session import PlatformSession

DEFAULT_EXPIRES_IN_SEC = MINUTES_10
DEFAULT_LATENCY_SEC = 20

delays = get_delays()


@dataclass
class TokenInfo:
    access_token: str = ""
    expires_in: Union[float, str] = 0
    expires_at: float = field(init=False)
    scope: Set[str] = field(default_factory=set)
    token_type: str = ""
    refresh_token: str = ""

    def __post_init__(self):
        self.expires_in = self.calc_expires_in(self.expires_in)
        self.expires_at = time.time() + self.expires_in

    @staticmethod
    def calc_expires_in(expires_in: Union[float, str]) -> float:
        try:
            expires_in = float(expires_in)
        except ValueError:
            expires_in = DEFAULT_EXPIRES_IN_SEC
        return max(expires_in, DEFAULT_EXPIRES_IN_SEC)

    def calc_token_update_time(self, latency: float) -> int:
        latency = min(latency, DEFAULT_LATENCY_SEC)

        expires_secs = self.expires_in / 2

        if expires_secs - latency > 0:
            expires_secs = expires_secs - latency

        return int(expires_secs)

    @classmethod
    def from_dict(cls, json_content: dict) -> "TokenInfo":
        return cls(
            access_token=json_content["access_token"],
            refresh_token=json_content["refresh_token"],
            expires_in=json_content.get("expires_in", ""),
            scope=set(json_content.get("scope", "").split()),
            token_type=json_content.get("token_type", ""),
        )

    def update(self, other: "TokenInfo") -> None:
        self.access_token = other.access_token
        self.refresh_token = other.refresh_token
        self.expires_in = other.expires_in
        self.expires_at = other.expires_at
        self.scope = other.scope
        self.token_type = other.token_type


class AuthManager(LogReporter):
    """
    Methods
    -------
    is_closed()
        The method returns True if closed, otherwise False

    is_authorized()
        The method returns True if authorized, otherwise False.
        If instance destroyed or closed always returns False

    authorize()
        The method starts process authorization

    close()
        The method stops refresh token updater and access token updater

    dispose()
        The method destroy an instance

    """

    def __init__(self, session: "PlatformSession", auto_reconnect: bool) -> None:
        LogReporter.__init__(self, logger=session)

        self._session = session
        self._auto_reconnect = auto_reconnect

        self._ee = CallbackHandler()
        self._token_info = TokenInfo()

        self._closed: bool = False
        self._authorized: bool = False
        self._disposed: bool = False

        self._result_evt: threading.Event = threading.Event()
        self._start_evt: threading.Event = threading.Event()
        self._thread: threading.Thread = threading.Thread(
            target=self._do_authorize,
            name="AuthManager-Thread",
            daemon=True,
        )

    @cached_property
    def _access_token_updater(self) -> AccessTokenUpdater:
        return AccessTokenUpdater(
            self._session,
            0.00001,
            self._access_token_update_handler,
        )

    @cached_property
    def _refresh_token_updater(self) -> RefreshTokenUpdater:
        return RefreshTokenUpdater(
            self._session,
            self._token_info,
            0.00001,
            self._refresh_token_update_handler,
        )

    def on(self, event: str, listener: Callable):
        self._ee.on(event, listener)

    def is_closed(self) -> bool:
        """

        Returns
        -------
        bool
            True if closed, otherwise False
        """
        return self._closed is True

    def is_authorized(self) -> bool:
        """

        Returns
        -------
        bool
            True if authorized, otherwise False
        """
        if self.is_closed():
            return False

        return self._authorized is True

    def authorize(self) -> bool:
        """
        The method starts process authorization

        Returns
        -------
        bool
            True if authorized, otherwise False
        """
        if self._disposed:
            raise RuntimeError("AuthManager disposed")

        if self.is_authorized():
            return True

        self.debug("AuthManager: start authorize")

        self._closed = False
        self._authorized = False

        self._result_evt.clear()
        self._start_evt.set()

        if not self._thread.ident:
            self._thread.start()

        self._result_evt.wait()

        is_authorized = self.is_authorized()
        self.debug(f"AuthManager: end authorize, result {is_authorized}")
        return is_authorized

    def _do_authorize(self):
        while not self._disposed:

            self._start_evt.wait()

            if self._disposed:
                break

            self.debug(
                f"AuthManager: Access token will be requested "
                f"in {self._access_token_updater.delay} seconds"
            )
            self._access_token_updater.start()

            if self.is_authorized():
                latency_secs = self._access_token_updater.latency_secs
                delay = self._token_info.calc_token_update_time(latency_secs)
                self._refresh_token_updater.delay = delay
                self.debug(
                    f"AuthManager: Refresh token will be requested "
                    f"in {self._refresh_token_updater.delay} seconds"
                )
                self._refresh_token_updater.start()

    def close(self):
        """
        The method stops refresh token updater and access token updater

        Returns
        -------
        None
        """
        if self._disposed:
            raise RuntimeError("AuthManager disposed")

        if self.is_closed():
            return

        self.debug("AuthManager: close")
        self._ee.emit(UpdateEvent.CLOSE_AUTH_MANAGER, self)
        self._start_evt.clear()
        self._access_token_updater.stop()
        self._refresh_token_updater.stop()
        self._closed = True
        self._authorized = False

    def _access_token_update_handler(
        self, event: str, message: str, json_content: dict
    ) -> None:
        self.debug(
            f"AuthManager: Access token handler, event: {event}, message: {message}"
        )

        if event is UpdateEvent.ACCESS_TOKEN_SUCCESS:
            self._authorized = True
            delays.reset()
            token_info = TokenInfo.from_dict(json_content)
            self._token_info.update(token_info)
            access_token = self._token_info.access_token
            self.debug(
                f"Access token {access_token}. "
                f"Expire in {self._token_info.expires_in} seconds"
            )
            self._ee.emit(UpdateEvent.UPDATE_ACCESS_TOKEN, access_token)
            self._access_token_updater.stop()
            self._ee.emit(event, message)
            self._ee.emit(UpdateEvent.AUTHENTICATION_SUCCESS, message)
            self._result_evt.set()

        elif event is UpdateEvent.ACCESS_TOKEN_UNAUTHORIZED:
            self._authorized = False
            self._access_token_updater.stop()
            self._ee.emit(event, message)
            self._ee.emit(UpdateEvent.AUTHENTICATION_FAILED, message)
            self.close()
            self._result_evt.set()

        elif event is UpdateEvent.ACCESS_TOKEN_FAILED:
            if not self._auto_reconnect:
                self._authorized = False
                self._access_token_updater.stop()
                self._result_evt.set()

            self._ee.emit(event, message)
            self._ee.emit(UpdateEvent.AUTHENTICATION_FAILED, message)

            if self._auto_reconnect:
                delay = delays.next()
                self.debug(f"AuthManager: reconnecting in {delay} secs")
                self._access_token_updater.delay = delay
                self._ee.emit(UpdateEvent.RECONNECTING, message)

            else:
                self.close()

    def _refresh_token_update_handler(
        self, event: str, message: str, json_content: dict
    ) -> None:
        self.debug(
            f"AuthManager: Refresh token handler, event: {event}, message: {message}"
        )

        if event is UpdateEvent.REFRESH_TOKEN_SUCCESS:
            token_info = TokenInfo.from_dict(json_content)
            self._token_info.update(token_info)
            access_token = token_info.access_token
            self.debug(
                f"Received access token {token_info.refresh_token}. "
                f"Expire in {token_info.expires_in} seconds"
            )
            self._ee.emit(UpdateEvent.UPDATE_ACCESS_TOKEN, access_token)
            latency_secs = self._access_token_updater.latency_secs
            delay = token_info.calc_token_update_time(latency_secs)
            self._refresh_token_updater.delay = delay
            self.debug(f"Set refresh token delay to {delay} seconds")
            self._ee.emit(event, message)

        elif event is UpdateEvent.REFRESH_TOKEN_BAD:
            self._authorized = False
            self._ee.emit(event, message)
            self._ee.emit(UpdateEvent.AUTHENTICATION_FAILED, message)
            self.close()

        elif event is UpdateEvent.REFRESH_TOKEN_FAILED:
            self._ee.emit(event, message)
            self._ee.emit(UpdateEvent.AUTHENTICATION_FAILED, message)

            if self._auto_reconnect:
                delay = delays.next()
                self.debug(
                    f"AuthManager: Trying to get Refresh token again in {delay} secs"
                )
                self._refresh_token_updater.delay = delay
                self._ee.emit(UpdateEvent.RECONNECTING, message)

            else:
                self._authorized = False
                self.close()

        elif event is UpdateEvent.REFRESH_TOKEN_EXPIRED:
            self._ee.emit(event, message)
            self._ee.emit(UpdateEvent.AUTHENTICATION_FAILED, message)
            self._authorized = False

            if self._auto_reconnect:
                self.debug("AuthManager: reconnecting")
                self._ee.emit(UpdateEvent.RECONNECTING, message)
                self._refresh_token_updater.stop()

            else:
                self.close()

    def dispose(self):
        """
        The method destroy an instance

        Returns
        -------
        None
        """
        if self._disposed:
            return

        if not self.is_closed():
            self.close()

        self._disposed = True
        self._access_token_updater.dispose()
        self._refresh_token_updater.dispose()
        self._start_evt.set()
        self._ee.remove_all_listeners()
        self._ee = None
        self._token_info = None
        self._start_evt = None
        self._thread = None
        self._session = None

    def verify_scope(self, key: str, method: str):
        required_scope_sets = SCOPE_BY_KEY.get((key.rstrip("/"), method.lower()))

        if required_scope_sets is None:
            self.debug(f"Scope for key={key}, method={method} not found.")

        if required_scope_sets and all(
            not required_scope.issubset(self._token_info.scope)
            for required_scope in required_scope_sets
        ):
            self.warning(f"No user scope for key={key}, method={method}.")
            raise ScopeError(required_scope_sets, self._token_info.scope, key, method)



# coding: utf8

from .._instrument_definition import InstrumentDefinition
from ._enums import BusinessDayConvention
from ._enums import DateRollingConvention
from ._enums import DayCountBasis


class TermDepositInstrumentDefinition(InstrumentDefinition):
    """
        API endpoint for Financial Contract analytics,
    that returns calculations relevant to each contract type.

    Parameters
    ----------
    instrument_tag : str, optional
        User defined string to identify the instrument. It can be used to link output
        results to the instrument definition. Only alphabetic, numeric and '- _.#=@'
        characters are supported.
    instrument_code : str, optional
        Code to define the term deposit instrument. For the moment, only RICs for CDs
        and Wholesales deposits are supported, with deposit code (ex:"EUR1MD=").
    start_date : str, optional
        The date the term deposit starts accruing interest. Its effective date.
        By default it is derived from the ValuationDate and the day to spot convention
        of the contract currency.
    end_date : str, optional
        The maturity date of the term deposit contract. Either the endDate or the tenor
        must be provided.
    tenor : str, optional
        The period code that represents the time between the start date and end date of
        the contract.
        Mandatory if instrumentCode is null. Either the endDate or the tenor must be
        provided.
    notional_ccy : str, optional
        The ISO code of the notional currency.
        Should be explicitly specified if InstrumentCode hasn't been specified.
        May be retrieved from reference data.
    notional_amount : float, optional
        The notional amount of the term deposit at the start date.
        By default 1,000,000 is used.
    fixed_rate_percent : float, optional
        Fixed interest rate percent to be applied for notional by deal terms.
        Mandatory if instrument_code is None.
    payment_business_day_convention : BusinessDayConvention, optional
        The method to adjust dates to a working day.
        By default 'ModifiedFollowing'.
    payment_roll_convention : DateRollingConvention, optional
        Method to adjust payment dates when they fall at the end of the month.
        By default 'Last'.
    year_basis : DayCountBasis, optional
        The Day Count Basis method used to calculate the interest payments.
        By default 'Dcb_Actual_365'.
    calendar : str, optional
        Calendar used to adjust deposit duration calculation.
        By default the calendar corresponding to notional currency is used.
    """

    def __init__(
        self,
        *,
        instrument_tag=None,
        instrument_code=None,
        start_date=None,
        end_date=None,
        tenor,
        notional_ccy,
        notional_amount=None,
        fixed_rate_percent=None,
        payment_business_day_convention=None,
        payment_roll_convention=None,
        year_basis=None,
        calendar=None,
    ):
        super().__init__()
        self.instrument_tag = instrument_tag
        self.instrument_code = instrument_code
        self.start_date = start_date
        self.end_date = end_date
        self.tenor = tenor
        self.notional_ccy = notional_ccy
        self.notional_amount = notional_amount
        self.fixed_rate_percent = fixed_rate_percent
        self.payment_business_day_convention = payment_business_day_convention
        self.payment_roll_convention = payment_roll_convention
        self.year_basis = year_basis
        self.calendar = calendar

    @classmethod
    def get_instrument_type(cls):
        return "TermDeposit"

    @property
    def payment_business_day_convention(self):
        """
        The method to adjust dates to a working day.
        The possible values are:
         - ModifiedFollowing (adjusts dates according to the Modified Following convention - next business day unless is it goes
         into the next month,
            preceeding is used in that  case),
         - NextBusinessDay (adjusts dates according to the Following convention - Next Business Day),
         - PreviousBusinessDay (adjusts dates  according to the Preceeding convention - Previous Business Day),
         - NoMoving (does not adjust dates),
         - BbswModifiedFollowing (adjusts dates  according to the BBSW Modified Following convention).
        Optional. In case an instrument code/style has been defined, value comes from bond reference data. Otherwise
        'ModifiedFollowing' is used.
        :return: enum BusinessDayConvention
        """
        return self._get_enum_parameter(
            BusinessDayConvention, "paymentBusinessDayConvention"
        )

    @payment_business_day_convention.setter
    def payment_business_day_convention(self, value):
        self._set_enum_parameter(
            BusinessDayConvention, "paymentBusinessDayConvention", value
        )

    @property
    def payment_roll_convention(self):
        """
        Method to adjust payment dates when they fall at the end of the month (28th of February, 30th, 31st).
        The possible values are:
         - Last (For setting the calculated date to the last working day),
         - Same (For setting the calculated date to the same day . In this latter case, the date may be moved according to the date
         moving
            convention if it is a non-working day),
         - Last28 (For setting the calculated date to the last working day. 28FEB being always considered as the last working day),
         - Same28 (For setting the calculated date to the same day .28FEB being always considered as the last working day).
        Optional. In case an instrument code has been defined, value comes from bond reference data. Otherwise, 'SameDay' is used.
        :return: enum DateRollingConvention
        """
        return self._get_enum_parameter(DateRollingConvention, "paymentRollConvention")

    @payment_roll_convention.setter
    def payment_roll_convention(self, value):
        self._set_enum_parameter(DateRollingConvention, "paymentRollConvention", value)

    @property
    def year_basis(self):
        """
        The Day Count Basis method used to calculate the interest payments.
        Dcb_Actual_365 used by default.
        :return: enum DayCountBasis
        """
        return self._get_enum_parameter(DayCountBasis, "yearBasis")

    @year_basis.setter
    def year_basis(self, value):
        self._set_enum_parameter(DayCountBasis, "yearBasis", value)

    @property
    def calendar(self):
        """
        Calendar used to adjust deposit duration calculation.
        By default the calendar corresponding to notional currency is used.
        :return: str
        """
        return self._get_parameter("calendar")

    @calendar.setter
    def calendar(self, value):
        self._set_parameter("calendar", value)

    @property
    def end_date(self):
        """
        The maturity date of the term deposit contract.
        Mandatory.
        Either the endDate or the tenor must be provided.
        :return: str
        """
        return self._get_parameter("endDate")

    @end_date.setter
    def end_date(self, value):
        self._set_parameter("endDate", value)

    @property
    def fixed_rate_percent(self):
        """
        Fixed interest rate percent to be applied for notional by deal terms.
        E.g. "10" means 10%
        :return: float
        """
        return self._get_parameter("fixedRatePercent")

    @fixed_rate_percent.setter
    def fixed_rate_percent(self, value):
        self._set_parameter("fixedRatePercent", value)

    @property
    def instrument_code(self):
        """
        Code to define the term deposit instrument.
        For the moment, only RICs for CDs and Wholesales deposits are supported, with deposit code (ex:"EUR1MD=").
        :return: str
        """
        return self._get_parameter("instrumentCode")

    @instrument_code.setter
    def instrument_code(self, value):
        self._set_parameter("instrumentCode", value)

    @property
    def instrument_tag(self):
        """
        User defined string to identify the instrument.
        It can be used to link output results to the instrument definition.
        Only alphabetic, numeric and '- _.#=@' characters are supported.
        Optional.
        :return: str
        """
        return self._get_parameter("instrumentTag")

    @instrument_tag.setter
    def instrument_tag(self, value):
        self._set_parameter("instrumentTag", value)

    @property
    def notional_amount(self):
        """
        The notional amount of the term deposit at the start date.
        Optional.
        By default 1,000,000 is used.
        :return: float
        """
        return self._get_parameter("notionalAmount")

    @notional_amount.setter
    def notional_amount(self, value):
        self._set_parameter("notionalAmount", value)

    @property
    def notional_ccy(self):
        """
        The ISO code of the notional currency.
        Should be explicitly specified if InstrumentCode hasn't been specified.
        May be retrieved from reference data.
        :return: str
        """
        return self._get_parameter("notionalCcy")

    @notional_ccy.setter
    def notional_ccy(self, value):
        self._set_parameter("notionalCcy", value)

    @property
    def start_date(self):
        """
        The date the term deposit starts accruing interest. Its effective date.
        Optional. By default it is derived from the ValuationDate and the day to spot convention of the contract currency.
        :return: str
        """
        return self._get_parameter("startDate")

    @start_date.setter
    def start_date(self, value):
        self._set_parameter("startDate", value)

    @property
    def tenor(self):
        """
        The period code that represents the time between the start date and end date of the contract.
        Mandatory if instrumentCode is null.
        Either the endDate or the tenor must be provided.
        Sample value: 1M
        :return: str
        """
        return self._get_parameter("tenor")

    @tenor.setter
    def tenor(self, value):
        self._set_parameter("tenor", value)



from refinitiv.data.errors import RDError
from refinitiv.data.content.ipa.curves import forward_curves
from tests.unit.conftest import StubSession, StubResponse


def test_forward_curves_two_errors_in_response_will_raise_error():
    # given
    extended_message = (
        "Error code QPS-Curves.6 | Invalid input: curveDefinition is missing"
    )
    response = StubResponse(
        {
            "data": [
                {
                    "error": {
                        "id": "babee8e8-6cd1-48bb-8e19-321c053ab17a/babee8e8-6cd1-48bb-8e19-321c053ab17a",
                        "code": "QPS-Curves.6",
                        "message": "Invalid input: curveDefinition is missing",
                    }
                },
                {
                    "error": {
                        "id": "babee8e8-6cd1-48bb-8e19-321c053ab17a/babee8e8-6cd1-48bb-8e19-321c053ab17a",
                        "code": "QPS-Curves.6",
                        "message": "Invalid input: curveDefinition is missing",
                    }
                },
            ]
        }
    )
    session = StubSession(is_open=True, response=response)
    definition = forward_curves.Definitions(
        [forward_curves.Definition(), forward_curves.Definition()]
    )

    try:
        # when
        definition.get_data(session)
    except RDError as e:
        # then
        assert str(e) == extended_message
    else:
        assert False


def test_forward_curves_one_error_in_response_will_not_raise_error():
    # given
    response = StubResponse(
        {
            "data": [
                {
                    "error": {
                        "id": "81124005-5a32-42fb-b833-5869ab04a1e8/81124005-5a32-42fb-b833-5869ab04a1e8",
                        "code": "QPS-Curves.6",
                        "message": "Invalid input: curveDefinition is missing",
                    }
                },
                {
                    "curveTag": "test_curve",
                    "curveParameters": {
                        "interestCalculationMethod": "Dcb_Actual_Actual",
                        "priceSide": "Mid",
                        "calendarAdjustment": "Calendar",
                        "calendars": ["EMU_FI"],
                        "compoundingType": "Compounded",
                        "useConvexityAdjustment": True,
                        "useSteps": False,
                        "valuationDate": "2022-02-08",
                    },
                    "curveDefinition": {
                        "availableTenors": ["OIS", "1M", "3M", "6M", "1Y"],
                        "availableDiscountingTenors": ["OIS", "1M", "3M", "6M", "1Y"],
                        "currency": "EUR",
                        "mainConstituentAssetClass": "Swap",
                        "riskType": "InterestRate",
                        "indexName": "EURIBOR",
                        "source": "Refinitiv",
                        "name": "EUR EURIBOR Swap ZC Curve",
                        "id": "9d619112-9ab3-45c9-b83c-eb04cbec382e",
                        "discountingTenor": "OIS",
                        "ignoreExistingDefinition": False,
                        "owner": "Refinitiv",
                    },
                    "forwardCurves": [
                        {
                            "curvePoints": [
                                {
                                    "endDate": "2021-02-01",
                                    "startDate": "2021-02-01",
                                    "discountFactor": 1.0,
                                    "ratePercent": -0.5576014785508066,
                                    "tenor": "0D",
                                },
                                {
                                    "endDate": "2021-02-04",
                                    "startDate": "2021-02-01",
                                    "discountFactor": 1.000045833991234,
                                    "ratePercent": -0.5576014785508066,
                                    "tenor": "1D",
                                },
                            ],
                            "forwardCurveTag": "ForwardTag",
                            "forwardStart": "2021-02-01",
                            "indexTenor": "3M",
                        }
                    ],
                },
            ]
        }
    )
    session = StubSession(is_open=True, response=response)
    definition = forward_curves.Definitions(
        [forward_curves.Definition(), forward_curves.Definition()]
    )

    try:
        # when
        response = definition.get_data(session)
    except RDError as e:
        # then
        assert False, str(e)
    else:
        assert len(response.errors) == 1



import pytest

from refinitiv.data._errors import RDError
from refinitiv.data.content.historical_pricing import events, summaries
from tests.unit.conftest import StubSession

from .data_for_tests import (
    DF_EMPTY_ONE_INST,
    DF_EMPTY_TWO_INSTS,
    DF_EMPTY_ONE_INST_IS_BAD,
    INST_IS_BAD,
    INSTS_ARE_BAD,
    DF_EMPTY_ONE_INST_IS_BAD_OTHER_WITHOUT_HEADERS,
    DF_EMPTY_WITHOUT_HEADERS,
)


@pytest.mark.parametrize(
    "universe,object_definition,response",
    [
        # events
        ("LSEG.L", events.Definition, DF_EMPTY_ONE_INST),
        (["LSEG.L", "VOD.L"], events.Definition, DF_EMPTY_TWO_INSTS),
        (["LSEG.L", "VOD.Lffff"], events.Definition, DF_EMPTY_ONE_INST_IS_BAD),
        # summaries
        ("LSEG.L", summaries.Definition, DF_EMPTY_ONE_INST),
        (["LSEG.L", "VOD.L"], summaries.Definition, DF_EMPTY_TWO_INSTS),
        (["LSEG.L", "VOD.Lffff"], summaries.Definition, DF_EMPTY_ONE_INST_IS_BAD),
    ],
)
def test_empty_df(universe, object_definition, response):
    # given
    session = StubSession(is_open=True, response=response)
    expected_str = "Empty DataFrame\n" "Columns: []\n" "Index: []"
    definition = object_definition(
        universe=universe,
        start="09-09-2020T16:30:00",
        end="09-09-2021T04:00:01",
    )
    # when
    response = definition.get_data(session=session)
    df = response.data.df

    # then
    assert df.empty
    assert df.to_string() == expected_str


@pytest.mark.parametrize(
    "universe, response",
    [
        (["EUR=", "GBP="], DF_EMPTY_WITHOUT_HEADERS),
        (["EUR=", "GBP"], DF_EMPTY_ONE_INST_IS_BAD_OTHER_WITHOUT_HEADERS),
    ],
)
def test_empty_df_without_headers_for_events_endpoints(universe, response):
    # given
    session = StubSession(is_open=True, response=response)
    expected_str = "Empty DataFrame\n" "Columns: []\n" "Index: []"
    definition = events.Definition(
        universe=universe,
        eventTypes="trade",
        start="09-09-1992T16:30:00",
        end="09-09-1993T04:00:01",
    )
    # when
    response = definition.get_data(session=session)
    df = response.data.df

    # then
    assert df.empty
    assert df.to_string() == expected_str


@pytest.mark.parametrize(
    "universe,object_definition,response",
    [
        # events
        ("VOD.Lffff", events.Definition, INST_IS_BAD),
        (["LSEG.Lffff", "VOD.Lffff"], events.Definition, INSTS_ARE_BAD),
        # summaries
        ("VOD.Lffff", summaries.Definition, INST_IS_BAD),
        (["LSEG.Lffff", "VOD.Lffff"], summaries.Definition, INSTS_ARE_BAD),
    ],
)
def test_raise_error(universe, object_definition, response):
    # given
    error_message = "Error code 1 | No data to return, please check errors: ERROR: No successful response."
    session = StubSession(is_open=True, response=response)
    definition = object_definition(universe=universe)
    # when
    with pytest.raises(expected_exception=RDError, match=error_message):
        definition.get_data(session=session)



from typing import TYPE_CHECKING

from ..delivery._data._data_provider import ContentValidator

if TYPE_CHECKING:
    from ..delivery._data._data_provider import ParsedData


class UniverseContentValidator(ContentValidator):
    @classmethod
    def content_data_has_no_error(cls, data: "ParsedData") -> bool:
        error = data.content_data.get("error", {})
        if error:
            data.error_codes = error.get("code")
            error_message = error.get("description")

            if error_message == "Unable to resolve all requested identifiers.":
                universe = data.raw_response.url.params["universe"].split(",")
                error_message = f"{error_message} Requested items: {universe}"

            if not error_message:
                error_message = error.get("message")
                errors = error.get("errors")

                if isinstance(errors, list):
                    errors = "\n".join(map(str, errors))
                    error_message = f"{error_message}:\n{errors}"

            data.error_messages = error_message

            return False

        invalid_universes = [
            universe.get("Instrument")
            for universe in data.content_data.get("universe", [])
            if universe.get("Organization PermID") == "Failed to resolve identifier(s)."
        ]
        if invalid_universes:
            data.error_messages = f"Failed to resolve identifiers {invalid_universes}"

        return True

    def __init__(self) -> None:
        super().__init__()
        self.validators.append(self.content_data_has_no_error)



from ..delivery._data._data_provider import Parser


class ErrorParser(Parser):
    def process_failed_response(self, raw_response):
        parsed_data = super().process_failed_response(raw_response)
        error_code = parsed_data.first_error_code
        error_message = parsed_data.first_error_message

        status = parsed_data.status
        error = status.get("error", {})
        errors = error.get("errors", [])

        err_msgs = []
        err_codes = []
        for err in errors:
            reason = err.get("reason")
            if reason:
                err_codes.append(error_code)
                err_msgs.append(f"{error_message}: {reason}")

        if err_msgs and err_codes:
            parsed_data.error_codes = err_codes
            parsed_data.error_messages = err_msgs

        return parsed_data



from typing import Optional

from ._field_formula_description import FieldFormulaDescription
from ...._object_definition import ObjectDefinition


class BidAskFieldsDescription(ObjectDefinition):
    """
    Creates the Cross Currency curve definition with the definition provided.

    Parameters
    ----------
    bid : FieldFormulaDescription, optional

    ask : FieldFormulaDescription, optional

    """

    def __init__(
        self,
        bid: Optional[FieldFormulaDescription] = None,
        ask: Optional[FieldFormulaDescription] = None,
    ) -> None:
        super().__init__()
        self.bid = bid
        self.ask = ask

    @property
    def ask(self):
        """
        :return: object FieldFormulaDescription
        """
        return self._get_object_parameter(FieldFormulaDescription, "ask")

    @ask.setter
    def ask(self, value):
        self._set_object_parameter(FieldFormulaDescription, "ask", value)

    @property
    def bid(self):
        """
        :return: object FieldFormulaDescription
        """
        return self._get_object_parameter(FieldFormulaDescription, "bid")

    @bid.setter
    def bid(self, value):
        self._set_object_parameter(FieldFormulaDescription, "bid", value)



__all__ = (
    "get_headlines",
    "get_story",
    "Urgency",
    "SortOrder",
    "Format",
)

from ._news import get_headlines, get_story, Format

from ...content.news._urgency import Urgency
from ...content.news._sort_order import SortOrder



from unittest.mock import patch, MagicMock

import pytest

from refinitiv.data.delivery._data._data_provider import ParsedData
from refinitiv.data.delivery.cfs._cfs_data_provider import (
    cfs_packages_data_provider,
    CFSRequestFactory,
    CFSStreamRequestFactory,
    CFSResponseFactory,
    CFSPackageRequestFactory,
    cfs_buckets_data_provider,
    cfs_file_sets_data_provider,
    cfs_files_data_provider,
)
from refinitiv.data.delivery.cfs._data_types import BaseData


@pytest.mark.parametrize(
    ("test_value", "expected"),
    [
        (
            {
                "url_root": "site.com",
                "url": "test_url",
                "path_parameters": [],
                "query_parameters": (("attributes", "size;depth"),),
            },
            "site.com/test_url?attributes=size;depth",
        ),
        (
            {
                "url_root": "site.com",
                "url": "test_url",
                "path_parameters": [],
                "query_parameters": tuple(),
            },
            "site.com/test_url",
        ),
    ],
)
def test_cfs_request_factory_update_url(test_value, expected):
    # given
    request_factory = CFSRequestFactory()

    # when
    result = request_factory.update_url(**test_value)

    # then
    assert result == expected


@pytest.mark.parametrize(
    ("test_value", "expected"),
    [
        (
            [..., "site.com/files"],
            "site.com/files/{id}/stream",
        ),
    ],
)
def test_cfs_stream_request_factory_get_url(test_value, expected):
    # given
    request_factory = CFSStreamRequestFactory()

    # when
    result = request_factory.get_url(*test_value)

    # then
    assert result == expected


@pytest.mark.parametrize(
    ("test_value", "expected"),
    [
        ({"_package_id": "5"}, "/file-store/v1/packages/5"),
        ({"_package_id": ""}, "/file-store/v1/packages/"),
        ({"_package_id": None}, "/file-store/v1/packages"),
        ({"package_id": "5"}, "/file-store/v1/packages"),
        ({"some_other": "5"}, "/file-store/v1/packages"),
    ],
)
def test_cfs_package_request_factory_get_url(test_value, expected):
    # given
    args = [..., "/file-store/v1/packages"]
    request_factory = CFSPackageRequestFactory()

    # when
    result = request_factory.get_url(*args, **test_value)

    # then
    assert result == expected


@pytest.mark.parametrize(
    "test_value",
    [
        {"_package_id": 5, "a": "aa"},
        {"_package_id": "5", "a": "aa"},
        {"_package_id": "", "a": "aa"},
    ],
)
def test_cfs_package_request_factory_get_query_parameters_with_id_argument(test_value):
    # given
    request_factory = CFSPackageRequestFactory()

    # when
    result = request_factory.get_query_parameters(**test_value)

    # then
    assert not result


@pytest.mark.parametrize(
    "test_value",
    [
        {"_package_id": None, "a": "aa"},
        {"package_id": "5", "a": "aa"},
    ],
)
def test_cfs_package_request_factory_get_query_parameters_not_valid_id_arg(test_value):
    # given
    request_factory = CFSPackageRequestFactory()

    # when
    result = request_factory.get_query_parameters(**test_value)

    # then
    assert result


@pytest.mark.parametrize(
    ("test_value", "expected"),
    [
        (
            {"path_parameters": {"par1": "val1", "par2": "val2"}},
            {"par1": "val1", "par2": "val2"},
        ),
        (
            {"path_parameters": {"par1": "val1", "par2": "val2"}, "id": "123"},
            {"par1": "val1", "par2": "val2", "id": "123"},
        ),
        (
            {"path_parameters": {"par1": "val1", "par2": "val2"}, "par": "123"},
            {"par1": "val1", "par2": "val2"},
        ),
        (
            {
                "path_parameters": {"par1": "val1", "par2": "val2", "id": "12"},
                "id": "123",
            },
            {"par1": "val1", "par2": "val2", "id": "123"},
        ),
    ],
)
def test_cfs_stream_request_factory_get_path_parameters(test_value, expected):
    # given
    request_factory = CFSStreamRequestFactory()

    # when
    result = request_factory.get_path_parameters(**test_value)

    # then
    assert result == expected


@pytest.mark.parametrize(
    ("test_value", "expected"),
    [
        (dict(), [("doNotRedirect", "true")]),
        (
            {"query_parameters": [("par1", "val1")]},
            [("par1", "val1"), ("doNotRedirect", "true")],
        ),
    ],
)
def test_cfs_stream_request_factory_get_query_parameters(test_value, expected):
    # given
    request_factory = CFSStreamRequestFactory()

    # when
    result = request_factory.get_query_parameters(**test_value)

    # then
    assert result == expected


@pytest.mark.parametrize(
    ("test_value", "expected"),
    [
        (
            ParsedData({}, {}, {}),
            {"content_data": {}},
        ),
        (
            ParsedData({}, {}, {"value": [dict()]}),
            {"content_data": {"value": [dict()]}},
        ),
    ],
)
def test_cfs_response_factory_create_success(test_value, expected):
    # given
    response_factory = CFSResponseFactory(data_class=MagicMock)

    mock_iter_obj = patch("refinitiv.data.delivery.cfs._cfs_data_provider.IterObj")

    # when
    mock_iter_obj.start()
    result = response_factory.create_success(test_value)
    mock_iter_obj.stop()

    # then
    assert result


def test_cfs_data_provider():
    # given
    # when
    for dp in (
        cfs_buckets_data_provider,
        cfs_file_sets_data_provider,
        cfs_files_data_provider,
    ):
        # then
        assert isinstance(dp.response, CFSResponseFactory)
        assert isinstance(dp.request, CFSRequestFactory)


def test_cfs_packages_data_provider():
    # when
    dp = cfs_packages_data_provider

    # then
    assert isinstance(dp.response, CFSResponseFactory)
    assert isinstance(dp.request, CFSPackageRequestFactory)


@pytest.mark.parametrize(
    "input_raw",
    [
        {"packageId": "0002-0002-00000002-0002-000000000002"},
        {
            "value": [
                {"packageId": "0002-0002-00000002-0002-000000000002"},
                {"packageId": "4000-0983-edb8b42c-8b72-a5aba64abb88"},
                {"packageId": "4000-0f0a-b1348f7c-b0a0-1203c3ee5a0f"},
            ]
        },
    ],
)
def test_cfs_build_df_raws(input_raw):
    # when
    data = BaseData(input_raw)

    # then
    assert not data.df.empty



from os.path import normpath

import pytest

from refinitiv.data.delivery.cfs._tools import (
    _convert_name,
    _get_query_parameter,
    _get_query_params,
    _convert_date_time,
    path_join,
    remove_one_ext,
)


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        ("name", "name"),
        ("created_since", "createdSince"),
        ("modified_since", "modifiedSince"),
        ("available_from", "availableFrom"),
        ("available_to", "availableTo"),
        ("attributes", "attributes"),
        ("page_size", "pageSize"),
        ("skip_token", "skipToken"),
        ("bucket", "bucket"),
        ("package_id", "packageId"),
        ("status", "status"),
        ("content_from", "contentFrom"),
        ("content_to", "contentTo"),
        ("fileset_id", "filesetId"),
        ("filesetId", "filesetId"),
        ("filename", "filename"),
        ("package_name", "packageName"),
        ("package_type", "packageType"),
        ("bucket_name", "bucketName"),
        ("page", "page"),
        ("included_total_result", "includedTotalResult"),
        ("included_entitlement_result", "includedEntitlementResult"),
    ],
)
def test__convert_name(test_value, expected_result):
    # given

    # when
    result = _convert_name(test_value)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        ({}, []),
        ({"x": 1, "y": None}, [("x", 1)]),
        (
            {"fileset_id": "wef32", "some_value": True},
            [("filesetId", "wef32"), ("someValue", True)],
        ),
    ],
)
def test_get_query_params(test_value, expected_result):
    assert expected_result == _get_query_params(**test_value)


def test__get_query_parameter():
    # given
    test_values = (
        "doNotRedirect",
        "https://api.refinitiv.com/file-store/v1/files/{fileId}/stream?doNotRedirect=true",
    )
    expected_result = "true"

    # when
    result = _get_query_parameter(*test_values)

    # then
    assert result == expected_result


def test__convert_date_time():
    # given
    test_value = "05-20-2021"

    # when
    result = _convert_date_time(test_value)
    expected_result = "2021-05-20T00:00:00Z"

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    (
        (("test_folder", "filename"), normpath("test_folder/filename")),
        (("/test_folder", "filename"), normpath("/test_folder/filename")),
        (("test_folder/", "filename"), normpath("test_folder/filename")),
        (("test_folder", "filename/"), normpath("test_folder/filename")),
        (("/test_folder/", "filename"), normpath("/test_folder/filename")),
        (("/test_folder", "filename/"), normpath("/test_folder/filename")),
        (("/test_folder/", "filename/"), normpath("/test_folder/filename")),
    ),
)
def test_path_join(test_value, expected_result):
    assert path_join(*test_value) == expected_result


@pytest.mark.parametrize(
    "input_value, expected_value",
    [
        ("filename.ext_1.ext_2", "filename.ext_1"),
        ("filename.ext_1", "filename"),
        ("filename", "filename"),
    ],
)
def test_remove_one_ext(input_value, expected_value):
    # when
    testing_value = remove_one_ext(input_value)

    # then
    assert testing_value == expected_value



# coding: utf8

from enum import Enum, unique


@unique
class DividendType(Enum):
    FORECAST_TABLE = "ForecastTable"
    FORECAST_YIELD = "ForecastYield"
    FUTURES = "Futures"
    HISTORICAL_YIELD = "HistoricalYield"
    IMPLIED_TABLE = "ImpliedTable"
    IMPLIED_YIELD = "ImpliedYield"
    NONE = "None"



from refinitiv.data.content.ipa.curves._cross_currency_curves.definitions import manage
from tests.unit.conftest import StubResponse, StubSession


def test_delete_function():
    # given
    response = StubResponse({"status": "200"})
    session = StubSession(is_open=True, response=response)
    response = manage.delete(
        id="7bdb00f3-0a48-40be-ace2-6d3cfd0e8e59",
        session=session,
    )

    # then
    assert response.data.raw
    assert response.data.df.empty



# coding: utf8

from typing import Optional

from .._instrument_definition import InstrumentDefinition
from ._enums import (
    BuySell,
    DayCountBasis,
)
from ._repo_underlying_contract import UnderlyingContract


class RepoInstrumentDefinition(InstrumentDefinition):
    """
    API endpoint for Financial Contract analytics,
    that returns calculations relevant to each contract type.

    Parameters
    ----------
    instrument_tag : str, optional
        User defined string to identify the instrument.it can be used to link output
        results to the instrument definition. only alphabetic, numeric and '- _.#=@'
        characters are supported. optional.
    start_date : str, optional
        Start date of the repo, that means when the underlying security is exchanged.
        mandatory.
    end_date : str, optional
        End date of the repo, that means when the borrower repurchases the security
        back. either enddate or tenor field are requested.
    tenor : str, optional
        Tenor that defines the duration of the repo in case no enddate has been
        provided. in that case, enddate is computed from startdate and tenor. either
        enddate or tenor field are requested.
    buy_sell : BuySell, optional
        The indicator of the deal side. the possible values are:   buy: buying the repo,
        sell: selling the repo.  optional. the default value is "buy".
    day_count_basis : DayCountBasis, optional
        Day count basis convention to apply to the custom repo rate. optional,
        "dcb_actual_360" by default.
    underlying_instruments : RepoUnderlyingContract, optional
        Definition of the underlying instruments. only bond contracts are supported for
        now, and only one bond can be used. mandatory.
    is_coupon_exchanged : bool, optional
        Specifies whether or not intermediate coupons are exchanged.
        - couponexchanged = true to specify that intermediate coupons for the underlying
          bond (between the repo start date and repo end date) are exchanged between the
          repo seller and repo buyer.
        - couponexchanged = false to specify that no intermediate coupons are exchanged
          between the repo seller and repo buyer. in this case the repo instrument is
          like a standard loan with no intermediate coupons; the bond is only used as a
          warranty in case the money borrower defaults. optional. true by default, which
          means coupon exchanged.
    repo_rate_percent : float, optional
        Custom repo rate in percentage. if not provided in the request, it will be
        computed by interpolating/extrapolating a repo curve. optional.
    """

    def __init__(
        self,
        instrument_tag: Optional[str] = None,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        tenor: Optional[str] = None,
        buy_sell: Optional[BuySell] = None,
        day_count_basis: Optional[DayCountBasis] = None,
        underlying_instruments: Optional[UnderlyingContract] = None,
        is_coupon_exchanged: Optional[bool] = None,
        repo_rate_percent: Optional[float] = None,
    ) -> None:
        super().__init__()
        self.instrument_tag = instrument_tag
        self.start_date = start_date
        self.end_date = end_date
        self.tenor = tenor
        self.buy_sell = buy_sell
        self.day_count_basis = day_count_basis
        self.underlying_instruments = underlying_instruments
        self.is_coupon_exchanged = is_coupon_exchanged
        self.repo_rate_percent = repo_rate_percent

    @classmethod
    def get_instrument_type(cls):
        return "Repo"

    @property
    def buy_sell(self):
        """
        The indicator of the deal side. the possible values are:   buy: buying the repo,
        sell: selling the repo.  optional. the default value is "buy".
        :return: enum BuySell
        """
        return self._get_enum_parameter(BuySell, "buySell")

    @buy_sell.setter
    def buy_sell(self, value):
        self._set_enum_parameter(BuySell, "buySell", value)

    @property
    def day_count_basis(self):
        """
        Day count basis convention to apply to the custom repo rate. optional,
        "dcb_actual_360" by default.
        :return: enum DayCountBasis
        """
        return self._get_enum_parameter(DayCountBasis, "dayCountBasis")

    @day_count_basis.setter
    def day_count_basis(self, value):
        self._set_enum_parameter(DayCountBasis, "dayCountBasis", value)

    @property
    def underlying_instruments(self):
        """
        Definition of the underlying instruments. only bond contracts are supported for
        now, and only one bond can be used. mandatory.
        :return: list RepoUnderlyingContract
        """
        return self._get_list_parameter(UnderlyingContract, "underlyingInstruments")

    @underlying_instruments.setter
    def underlying_instruments(self, value):
        self._set_list_parameter(UnderlyingContract, "underlyingInstruments", value)

    @property
    def end_date(self):
        """
        End date of the repo, that means when the borrower repurchases the security
        back. either enddate or tenor field are requested.
        :return: str
        """
        return self._get_parameter("endDate")

    @end_date.setter
    def end_date(self, value):
        self._set_parameter("endDate", value)

    @property
    def instrument_tag(self):
        """
        User defined string to identify the instrument.it can be used to link output
        results to the instrument definition. only alphabetic, numeric and '- _.#=@'
        characters are supported. optional.
        :return: str
        """
        return self._get_parameter("instrumentTag")

    @instrument_tag.setter
    def instrument_tag(self, value):
        self._set_parameter("instrumentTag", value)

    @property
    def is_coupon_exchanged(self):
        """
        Specifies whether or not intermediate coupons are exchanged.
        - couponexchanged = true to specify that intermediate coupons for the underlying
          bond (between the repo start date and repo end date) are exchanged between the
          repo seller and repo buyer.
        - couponexchanged = false to specify that no intermediate coupons are exchanged
          between the repo seller and repo buyer. in this case the repo instrument is
          like a standard loan with no intermediate coupons; the bond is only used as a
          warranty in case the money borrower defaults. optional. true by default, which
          means coupon exchanged.
        :return: bool
        """
        return self._get_parameter("isCouponExchanged")

    @is_coupon_exchanged.setter
    def is_coupon_exchanged(self, value):
        self._set_parameter("isCouponExchanged", value)

    @property
    def repo_rate_percent(self):
        """
        Custom repo rate in percentage. if not provided in the request, it will be
        computed by interpolating/extrapolating a repo curve. optional.
        :return: float
        """
        return self._get_parameter("repoRatePercent")

    @repo_rate_percent.setter
    def repo_rate_percent(self, value):
        self._set_parameter("repoRatePercent", value)

    @property
    def start_date(self):
        """
        Start date of the repo, that means when the underlying security is exchanged.
        mandatory.
        :return: str
        """
        return self._get_parameter("startDate")

    @start_date.setter
    def start_date(self, value):
        self._set_parameter("startDate", value)

    @property
    def tenor(self):
        """
        Tenor that defines the duration of the repo in case no enddate has been
        provided. in that case, enddate is computed from startdate and tenor. either
        enddate or tenor field are requested.
        :return: str
        """
        return self._get_parameter("tenor")

    @tenor.setter
    def tenor(self, value):
        self._set_parameter("tenor", value)



from refinitiv.data.content.ipa.curves._cross_currency_curves.triangulate_definitions import (
    search,
)


def test_search_definition_repr():
    # given
    definition = search.Definition()
    expected_value = (
        f"<refinitiv.data.content.ipa.curves._cross_currency_curves.triangulate_definitions.search.Definition "
        f"object at {hex(id(definition))}>"
    )

    # when
    testing_value = repr(definition)

    # then
    assert testing_value == expected_value
